@@ -217,29 +217,30 @@ void TestRunner::scheduleNext()
 void TestRunner::cancelCurrent(TestRunner::CancelReason reason)
 {
     m_canceled = true;
-    if (reason == UserCanceled) {
-        // when using the stop button we need to report, for progress bar this happens automatically
-        if (m_fakeFutureInterface && !m_fakeFutureInterface->isCanceled())
-            m_fakeFutureInterface->reportCanceled();
-    } else if (reason == KitChanged) {
-        if (m_fakeFutureInterface)
-            m_fakeFutureInterface->reportCanceled();
-        emit testResultReady(TestResultPtr(new FaultyTestResult(Result::MessageWarn,
-                tr("Current kit has changed. Canceling test run."))));
-    }
+
+    if (m_fakeFutureInterface)
+        m_fakeFutureInterface->reportCanceled();
+
+    auto reportResult = [this](Result::Type type, const QString &detail){
+        emit testResultReady(TestResultPtr(new FaultyTestResult(type, detail)));
+    };
+
+    if (reason == KitChanged)
+        reportResult(Result::MessageWarn, tr("Current kit has changed. Canceling test run."));
+    else if (reason == Timeout)
+        reportResult(Result::MessageFatal, tr("Test case canceled due to timeout.\nMaybe raise the timeout?"));
+
+    // if user or timeout cancels the current run ensure to kill the running process
     if (m_currentProcess && m_currentProcess->state() != QProcess::NotRunning) {
         m_currentProcess->kill();
         m_currentProcess->waitForFinished();
     }
-    if (reason == Timeout) {
-        emit testResultReady(TestResultPtr(new FaultyTestResult(Result::MessageFatal,
-                tr("Test case canceled due to timeout.\nMaybe raise the timeout?"))));
-    }
 }
 
 void TestRunner::onProcessFinished()
 {
-    if (m_currentConfig) {
+    if (m_executingTests && QTC_GUARD(m_currentConfig)) {
+        QTC_CHECK(m_fakeFutureInterface);
         m_fakeFutureInterface->setProgressValue(m_fakeFutureInterface->progressValue()
                                                 + m_currentConfig->testCaseCount());
         if (!m_fakeFutureInterface->isCanceled()) {
@@ -258,6 +259,10 @@ void TestRunner::onProcessFinished()
     }
     resetInternalPointers();
 
+    if (!m_fakeFutureInterface) {
+        QTC_ASSERT(!m_executingTests, m_executingTests = false);
+        return;
+    }
     if (!m_selectedTests.isEmpty() && !m_fakeFutureInterface->isCanceled())
         scheduleNext();
     else
@@ -553,8 +558,9 @@ void TestRunner::debugTests()
                 outputreader, &QObject::deleteLater);
     }
 
-    connect(this, &TestRunner::requestStopTestRun, runControl,
-            &ProjectExplorer::RunControl::initiateStop);
+    m_stopDebugConnect = connect(this, &TestRunner::requestStopTestRun,
+                                 runControl, &ProjectExplorer::RunControl::initiateStop);
+
     connect(runControl, &ProjectExplorer::RunControl::stopped, this, &TestRunner::onFinished);
     ProjectExplorer::ProjectExplorerPlugin::startRunControl(runControl);
 }
@@ -613,6 +619,7 @@ void TestRunner::onFinished()
     qDeleteAll(m_selectedTests);
     m_selectedTests.clear();
 
+    disconnect(m_stopDebugConnect);
     disconnect(m_targetConnect);
     m_fakeFutureInterface = nullptr;
     m_executingTests = false;
