@@ -130,7 +130,9 @@ TeaLeafReader::TeaLeafReader()
 {
     connect(EditorManager::instance(), &EditorManager::aboutToSave,
             this, [this](const IDocument *document) {
-        if (m_cmakeFiles.contains(document->filePath()) || !m_parameters.isAutorun)
+        if (m_cmakeFiles.contains(document->filePath())
+                || !m_parameters.cmakeTool
+                || !m_parameters.cmakeTool->isAutoRun())
             emit dirty();
     });
 
@@ -150,21 +152,20 @@ TeaLeafReader::~TeaLeafReader()
     resetData();
 }
 
-bool TeaLeafReader::isCompatible(const BuildDirReader::Parameters &p)
+bool TeaLeafReader::isCompatible(const BuildDirParameters &p)
 {
-    return !p.cmakeHasServerMode;
+    if (!p.cmakeTool)
+        return false;
+    return !p.cmakeTool->hasServerMode();
 }
 
 void TeaLeafReader::resetData()
 {
-    m_hasData = false;
-
     qDeleteAll(m_watchedFiles);
     m_watchedFiles.clear();
 
     m_projectName.clear();
     m_buildTargets.clear();
-    qDeleteAll(m_files);
     m_files.clear();
 }
 
@@ -188,18 +189,17 @@ static QString findCbpFile(const QDir &directory)
     return file;
 }
 
-void TeaLeafReader::parse(bool force)
+void TeaLeafReader::parse(bool forceConfiguration)
 {
-    const QString cbpFile = findCbpFile(QDir(m_parameters.buildDirectory.toString()));
+    const QString cbpFile = findCbpFile(QDir(m_parameters.workDirectory.toString()));
     const QFileInfo cbpFileFi = cbpFile.isEmpty() ? QFileInfo() : QFileInfo(cbpFile);
-    if (!cbpFileFi.exists()) {
+    if (!cbpFileFi.exists() || forceConfiguration) {
         // Initial create:
         startCMake(toArguments(m_parameters.configuration, m_parameters.expander));
         return;
     }
 
-    const bool mustUpdate = force
-            || m_cmakeFiles.isEmpty()
+    const bool mustUpdate = m_cmakeFiles.isEmpty()
             || anyOf(m_cmakeFiles, [&cbpFileFi](const FileName &f) {
                    return f.toFileInfo().lastModified() > cbpFileFi.lastModified();
                });
@@ -207,7 +207,6 @@ void TeaLeafReader::parse(bool force)
         startCMake(QStringList());
     } else {
         extractData();
-        m_hasData = true;
         emit dataAvailable();
     }
 }
@@ -229,22 +228,21 @@ bool TeaLeafReader::isParsing() const
     return m_cmakeProcess && m_cmakeProcess->state() != QProcess::NotRunning;
 }
 
-bool TeaLeafReader::hasData() const
-{
-    return m_hasData;
-}
-
-QList<CMakeBuildTarget> TeaLeafReader::buildTargets() const
+QList<CMakeBuildTarget> TeaLeafReader::takeBuildTargets()
 {
     return m_buildTargets;
 }
 
 CMakeConfig TeaLeafReader::takeParsedConfiguration()
 {
-    FileName cacheFile = m_parameters.buildDirectory;
+    FileName cacheFile = m_parameters.workDirectory;
     cacheFile.appendPath(QLatin1String("CMakeCache.txt"));
+
+    if (!cacheFile.exists())
+        return { };
+
     QString errorMessage;
-    CMakeConfig result = BuildDirManager::parseConfiguration(cacheFile, &errorMessage);
+    CMakeConfig result = BuildDirManager::parseCMakeConfiguration(cacheFile, &errorMessage);
 
     if (!errorMessage.isEmpty()) {
         emit errorOccured(errorMessage);
@@ -266,11 +264,14 @@ CMakeConfig TeaLeafReader::takeParsedConfiguration()
 
 void TeaLeafReader::generateProjectTree(CMakeProjectNode *root, const QList<const FileNode *> &allFiles)
 {
+    if (m_files.size() == 0)
+        return;
+
     root->setDisplayName(m_projectName);
 
     // Delete no longer necessary file watcher based on m_cmakeFiles:
     const QSet<FileName> currentWatched
-            = transform(m_watchedFiles, [](CMakeFile *cmf) { return cmf->filePath(); });
+            = transform(m_watchedFiles, &CMakeFile::filePath);
     const QSet<FileName> toWatch = m_cmakeFiles;
     QSet<FileName> toDelete = currentWatched;
     toDelete.subtract(toWatch);
@@ -310,18 +311,20 @@ void TeaLeafReader::generateProjectTree(CMakeProjectNode *root, const QList<cons
     });
 
     // filter duplicates:
-    auto alreadySeen = QSet<FileName>::fromList(Utils::transform(m_files, &FileNode::filePath));
+    auto alreadySeen = Utils::transform<QSet>(m_files, &FileNode::filePath);
     const QList<const FileNode *> unseenMissingHeaders = Utils::filtered(missingHeaders, [&alreadySeen](const FileNode *fn) {
         const int count = alreadySeen.count();
         alreadySeen.insert(fn->filePath());
         return (alreadySeen.count() != count);
     });
 
-    const QList<FileNode *> fileNodes = m_files
-            + Utils::transform(unseenMissingHeaders, [](const FileNode *fn) { return fn->clone(); });
+    root->addNestedNodes(std::move(m_files), m_parameters.sourceDirectory);
 
-    root->addNestedNodes(fileNodes, m_parameters.sourceDirectory);
-    m_files.clear(); // Some of the FileNodes in files() were deleted!
+    std::vector<std::unique_ptr<FileNode>> fileNodes
+            = transform<std::vector>(unseenMissingHeaders, [](const FileNode *fn) {
+        return std::unique_ptr<FileNode>(fn->clone());
+    });
+    root->addNestedNodes(std::move(fileNodes), m_parameters.sourceDirectory);
 }
 
 static void processCMakeIncludes(const CMakeBuildTarget &cbt, const ToolChain *tc,
@@ -364,10 +367,10 @@ void TeaLeafReader::updateCodeModel(CppTools::RawProjectParts &rpps)
         } else {
             includePaths = transform(cbt.includeFiles, &FileName::toString);
         }
-        includePaths += m_parameters.buildDirectory.toString();
+        includePaths += m_parameters.workDirectory.toString();
         CppTools::RawProjectPart rpp;
-        rpp.setProjectFileLocation(QString()); // No project file information available!
-        rpp.setBuildSystemTarget(cbt.title);
+        rpp.setProjectFileLocation(cbt.sourceDirectory.toString() + "/CMakeLists.txt");
+        rpp.setBuildSystemTarget(cbt.title + QChar('\n') + cbt.sourceDirectory.toString() + QChar('/'));
         rpp.setIncludePaths(includePaths);
 
         CppTools::RawProjectPartFlags cProjectFlags;
@@ -378,10 +381,13 @@ void TeaLeafReader::updateCodeModel(CppTools::RawProjectParts &rpps)
         cxxProjectFlags.commandLineFlags = cxxflags;
         rpp.setFlagsForCxx(cxxProjectFlags);
 
-        rpp.setDefines(cbt.defines);
+        rpp.setMacros(cbt.macros);
         rpp.setDisplayName(cbt.title);
-        rpp.setFiles(transform(cbt.files, [](const FileName &fn) { return fn.toString(); }));
+        rpp.setFiles(transform(cbt.files, &FileName::toString));
 
+        const bool isExecutable = cbt.targetType == ExecutableType;
+        rpp.setBuildTargetType(isExecutable ? CppTools::ProjectPart::Executable
+                                            : CppTools::ProjectPart::Library);
         rpps.append(rpp);
     }
 }
@@ -403,14 +409,16 @@ void TeaLeafReader::cleanUpProcess()
 
 void TeaLeafReader::extractData()
 {
+    QTC_ASSERT(m_parameters.isValid() && m_parameters.cmakeTool, return);
+
     const FileName srcDir = m_parameters.sourceDirectory;
-    const FileName bldDir = m_parameters.buildDirectory;
+    const FileName bldDir = m_parameters.workDirectory;
     const FileName topCMake = Utils::FileName(srcDir).appendPath("CMakeLists.txt");
 
     resetData();
 
     m_projectName = m_parameters.projectName;
-    m_files.append(new FileNode(topCMake, FileType::Project, false));
+    m_files.emplace_back(std::make_unique<FileNode>(topCMake, FileType::Project, false));
     // Do not insert topCMake into m_cmakeFiles: The project already watches that!
 
     // Find cbp file
@@ -420,7 +428,7 @@ void TeaLeafReader::extractData()
     m_cmakeFiles.insert(cbpFile);
 
     // Add CMakeCache.txt file:
-    FileName cacheFile = m_parameters.buildDirectory;
+    FileName cacheFile = m_parameters.workDirectory;
     cacheFile.appendPath(QLatin1String("CMakeCache.txt"));
     if (cacheFile.toFileInfo().exists())
         m_cmakeFiles.insert(cacheFile);
@@ -428,34 +436,38 @@ void TeaLeafReader::extractData()
     // setFolderName
     CMakeCbpParser cbpparser;
     // Parsing
-    if (!cbpparser.parseCbpFile(m_parameters.pathMapper, cbpFile, srcDir))
+    if (!cbpparser.parseCbpFile(m_parameters.cmakeTool->pathMapper(), cbpFile, srcDir))
         return;
 
     m_projectName = cbpparser.projectName();
 
-    m_files = cbpparser.fileList();
+    m_files = cbpparser.takeFileList();
     if (cbpparser.hasCMakeFiles()) {
-        m_files.append(cbpparser.cmakeFileList());
-        foreach (const FileNode *node, cbpparser.cmakeFileList())
+        std::vector<std::unique_ptr<FileNode>> cmakeNodes = cbpparser.takeCmakeFileList();
+        for (const std::unique_ptr<FileNode> &node : cmakeNodes)
             m_cmakeFiles.insert(node->filePath());
+
+        std::move(std::begin(cmakeNodes), std::end(cmakeNodes), std::back_inserter(m_files));
     }
 
     // Make sure the top cmakelists.txt file is always listed:
-    if (!contains(m_files, [topCMake](FileNode *fn) { return fn->filePath() == topCMake; }))
-        m_files.append(new FileNode(topCMake, FileType::Project, false));
-
-    Utils::sort(m_files, &Node::sortByPath);
+    if (!contains(m_files, [topCMake](const std::unique_ptr<FileNode> &fn) {
+                      return fn->filePath() == topCMake;
+                  }))
+        m_files.emplace_back(std::make_unique<FileNode>(topCMake, FileType::Project, false));
 
     m_buildTargets = cbpparser.buildTargets();
 }
 
 void TeaLeafReader::startCMake(const QStringList &configurationArguments)
 {
-    const FileName buildDirectory = m_parameters.buildDirectory;
+    QTC_ASSERT(m_parameters.isValid() && m_parameters.cmakeTool, return);
+
+    const FileName workDirectory = m_parameters.workDirectory;
     QTC_ASSERT(!m_cmakeProcess, return);
     QTC_ASSERT(!m_parser, return);
     QTC_ASSERT(!m_future, return);
-    QTC_ASSERT(buildDirectory.exists(), return);
+    QTC_ASSERT(workDirectory.exists(), return);
 
     const QString srcDir = m_parameters.sourceDirectory.toString();
 
@@ -476,7 +488,7 @@ void TeaLeafReader::startCMake(const QStringList &configurationArguments)
     // then we are racing against CMakeCache.txt also getting deleted.
 
     m_cmakeProcess = new QtcProcess;
-    m_cmakeProcess->setWorkingDirectory(buildDirectory.toString());
+    m_cmakeProcess->setWorkingDirectory(workDirectory.toString());
     m_cmakeProcess->setEnvironment(m_parameters.environment);
 
     connect(m_cmakeProcess, &QProcess::readyReadStandardOutput,
@@ -494,9 +506,9 @@ void TeaLeafReader::startCMake(const QStringList &configurationArguments)
     TaskHub::clearTasks(ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM);
 
     MessageManager::write(tr("Running \"%1 %2\" in %3.")
-                          .arg(m_parameters.cmakeExecutable.toUserOutput())
+                          .arg(m_parameters.cmakeTool->cmakeExecutable().toUserOutput())
                           .arg(args)
-                          .arg(buildDirectory.toUserOutput()));
+                          .arg(workDirectory.toUserOutput()));
 
     m_future = new QFutureInterface<void>();
     m_future->setProgressRange(0, 1);
@@ -504,7 +516,7 @@ void TeaLeafReader::startCMake(const QStringList &configurationArguments)
                              tr("Configuring \"%1\"").arg(m_parameters.projectName),
                              "CMake.Configure");
 
-    m_cmakeProcess->setCommand(m_parameters.cmakeExecutable.toString(), args);
+    m_cmakeProcess->setCommand(m_parameters.cmakeTool->cmakeExecutable().toString(), args);
     emit configurationStarted();
     m_cmakeProcess->start();
 }
@@ -540,7 +552,6 @@ void TeaLeafReader::cmakeFinished(int code, QProcess::ExitStatus status)
     delete m_future;
     m_future = nullptr;
 
-    m_hasData = true;
     emit dataAvailable();
 }
 
@@ -548,7 +559,7 @@ void TeaLeafReader::processCMakeOutput()
 {
     static QString rest;
     rest = lineSplit(rest, m_cmakeProcess->readAllStandardOutput(),
-                     [this](const QString &s) { MessageManager::write(s); });
+                     [](const QString &s) { MessageManager::write(s); });
 }
 
 void TeaLeafReader::processCMakeError()
@@ -654,7 +665,7 @@ bool TeaLeafReader::extractFlagsFromNinja(const CMakeBuildTarget &buildTarget,
     // found
     // Get "all" target's working directory
     QByteArray ninjaFile;
-    QString buildNinjaFile = buildTargets().at(0).workingDirectory.toString();
+    QString buildNinjaFile = takeBuildTargets().at(0).workingDirectory.toString();
     buildNinjaFile += "/build.ninja";
     QFile buildNinja(buildNinjaFile);
     if (buildNinja.exists()) {
