@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef CPPCODESTYLESETTINGS_H
-#define CPPCODESTYLESETTINGS_H
+#pragma once
 
 #include "cpptools_global.h"
 
@@ -81,6 +80,8 @@ public:
     //            b
     bool alignAssignments;
 
+    bool preferGetterNameWithoutGetPrefix;
+
     void toSettings(const QString &category, QSettings *s) const;
     void fromSettings(const QString &category, const QSettings *s);
 
@@ -91,6 +92,9 @@ public:
     bool operator==(const CppCodeStyleSettings &s) const { return equals(s); }
     bool operator!=(const CppCodeStyleSettings &s) const { return !equals(s); }
 
+    static CppCodeStyleSettings currentProjectCodeStyle();
+    static CppCodeStyleSettings currentGlobalCodeStyle();
+
     /*! Returns an Overview configured by the current project's code style.
 
         If no current project is available or an error occurs when getting the
@@ -110,5 +114,3 @@ public:
 } // namespace CppTools
 
 Q_DECLARE_METATYPE(CppTools::CppCodeStyleSettings)
-
-#endif // CPPCODESTYLESETTINGS_H
