@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,25 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "sourceutils.h"
 
+#include "debuggerinternalconstants.h"
+#include "debuggerengine.h"
+#include "disassemblerlines.h"
 #include "watchdata.h"
 #include "watchutils.h"
 
@@ -45,6 +43,8 @@
 #include <utils/qtcassert.h>
 
 #include <QDebug>
+#include <QTextDocument>
+#include <QTextBlock>
 
 #include <string.h>
 #include <ctype.h>
@@ -174,7 +174,7 @@ static void blockRecursion(const Overview &overview,
             // Is the declaration on or past the current line, that is,
             // the variable not initialized.
             if (symbol->line() >= line)
-                uninitializedVariables->push_back(WatchData::shadowedName(name, it.value()));
+                uninitializedVariables->push_back(WatchItem::shadowedName(name, it.value()));
         }
     }
     // Next block scope.
@@ -205,8 +205,8 @@ int getUninitializedVariablesI(const Snapshot &snapshot,
         return 4;
     // First figure out the function to do a safety name check
     // and the innermost scope at cursor position
-    const Function *function = 0;
-    const Scope *innerMostScope = 0;
+    const Function *function = nullptr;
+    const Scope *innerMostScope = nullptr;
     if (symbolAtLine->isFunction()) {
         function = symbolAtLine->asFunction();
         if (function->memberCount() == 1) // Skip over function block
@@ -333,5 +333,37 @@ QString fixCppExpression(const QString &expIn)
     return removeObviousSideEffects(exp);
 }
 
+ContextData getLocationContext(TextDocument *document, int lineNumber)
+{
+    ContextData data;
+    QTC_ASSERT(document, return data);
+    if (document->property(Constants::OPENED_WITH_DISASSEMBLY).toBool()) {
+        QString line = document->document()->findBlockByNumber(lineNumber - 1).text();
+        DisassemblerLine l;
+        l.fromString(line);
+        if (l.address) {
+            data.type = LocationByAddress;
+            data.address = l.address;
+        } else {
+            QString fileName = document->property(Constants::DISASSEMBLER_SOURCE_FILE).toString();
+            if (!fileName.isEmpty()) {
+                // Possibly one of the  "27 [1] foo = x" lines
+                int pos = line.indexOf(QLatin1Char('['));
+                int ln = line.leftRef(pos - 1).toInt();
+                if (ln > 0) {
+                    data.type = LocationByFile;
+                    data.fileName = fileName;
+                    data.lineNumber = ln;
+                }
+            }
+        }
+    } else {
+        data.type = LocationByFile;
+        data.fileName = document->filePath().toString();
+        data.lineNumber = lineNumber;
+    }
+    return data;
+}
+
 } // namespace Internal
 } // namespace Debugger
