@@ -25,6 +25,7 @@
 
 #include "macwebkithelpviewer.h"
 
+#include "helpconstants.h"
 #include "localhelpmanager.h"
 #include "openpagesmanager.h"
 
@@ -168,7 +169,7 @@ static QPoint flipPoint(const NSPoint &p)
 
     NSURL *resolvedURL = data.resolvedUrl.toNSURL();
     NSString *mimeType = data.mimeType.toNSString();
-    NSData *nsdata = QtMac::toNSData(data.data); // Qt 5.3 has this in QByteArray
+    NSData *nsdata = data.data.toNSData();
     NSURLResponse *response = [[NSURLResponse alloc] initWithURL:resolvedURL
                                                         MIMEType:mimeType
                                            expectedContentLength:data.data.length()
@@ -201,6 +202,7 @@ static void ensureProtocolHandler()
 {
     WebFrame *mainFrame;
     Help::Internal::MacWebKitHelpViewer *viewer;
+    bool finished;
 }
 
 - (id)initWithMainFrame:(WebFrame *)frame viewer:(Help::Internal::MacWebKitHelpViewer *)viewer;
@@ -208,6 +210,7 @@ static void ensureProtocolHandler()
 - (void)webView:(WebView *)sender didReceiveTitle:(NSString *)title forFrame:(WebFrame *)frame;
 - (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame;
 - (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame;
+- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error forFrame:(WebFrame *)frame;
 
 @end
 
@@ -219,6 +222,7 @@ static void ensureProtocolHandler()
     if (self) {
         mainFrame = frame;
         viewer = helpViewer;
+        finished = false;
     }
     return self;
 }
@@ -226,8 +230,10 @@ static void ensureProtocolHandler()
 - (void)webView:(WebView *)sender didStartProvisionalLoadForFrame:(WebFrame *)frame
 {
     Q_UNUSED(sender)
-    if (frame == mainFrame)
+    if (frame == mainFrame) {
+        finished = false;
         viewer->slotLoadStarted();
+    }
 }
 
 - (void)webView:(WebView *)sender didReceiveTitle:(NSString *)title forFrame:(WebFrame *)frame
@@ -241,18 +247,31 @@ static void ensureProtocolHandler()
 - (void)webView:(WebView *)sender didFinishLoadForFrame:(WebFrame *)frame
 {
     Q_UNUSED(sender)
-    if (frame == mainFrame)
+    if (frame == mainFrame && !finished) {
+        finished = true;
         viewer->slotLoadFinished();
+    }
 }
 
 - (void)webView:(WebView *)sender didFailProvisionalLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
 {
     Q_UNUSED(sender)
     Q_UNUSED(error)
-    if (frame == mainFrame)
+    if (frame == mainFrame && !finished) {
+        finished = true;
         viewer->slotLoadFinished();
+    }
 }
 
+- (void)webView:(WebView *)sender didFailLoadWithError:(NSError *)error forFrame:(WebFrame *)frame
+{
+    Q_UNUSED(sender)
+    Q_UNUSED(error)
+    if (frame == mainFrame && !finished) {
+        finished = true;
+        viewer->slotLoadFinished();
+    }
+}
 @end
 
 // #pragma mark -- UIDelegate
@@ -262,15 +281,15 @@ static void ensureProtocolHandler()
     Help::Internal::MacWebKitHelpWidget *widget;
 }
 
-@property (assign) BOOL openInNewPageActionVisible;
-
 - (id)initWithWidget:(Help::Internal::MacWebKitHelpWidget *)theWidget;
 - (void)webView:(WebView *)sender makeFirstResponder:(NSResponder *)responder;
 - (NSArray *)webView:(WebView *)sender contextMenuItemsForElement:(NSDictionary *)element
     defaultMenuItems:(NSArray *)defaultMenuItems;
-- (WebView *)webView:(WebView *)sender createWebViewWithRequest:(NSURLRequest *)request;
 - (void)webView:(WebView *)sender mouseDidMoveOverElement:(NSDictionary *)elementInformation
     modifierFlags:(NSUInteger)modifierFlags;
+
+- (void)openAsNewPage:(id)sender;
+- (void)openInWindow:(id)sender;
 @end
 
 @implementation UIDelegate
@@ -280,7 +299,6 @@ static void ensureProtocolHandler()
     self = [super init];
     if (self) {
         widget = theWidget;
-        self.openInNewPageActionVisible = YES;
     }
     return self;
 }
@@ -294,6 +312,16 @@ static void ensureProtocolHandler()
     }
 }
 
+static NSMenuItem *menuItem(NSURL *url, id target, SEL action, const QString &title)
+{
+    NSMenuItem *openItem = [[NSMenuItem alloc] initWithTitle:title.toNSString()
+                                                      action:action
+                                               keyEquivalent:@""];
+    openItem.representedObject = url;
+    openItem.target = target;
+    return [openItem autorelease];
+}
+
 - (NSArray *)webView:(WebView *)sender contextMenuItemsForElement:(NSDictionary *)element
     defaultMenuItems:(NSArray *)defaultMenuItems
 {
@@ -317,11 +345,18 @@ static void ensureProtocolHandler()
             [ret addObject:item];
             break;
         case WebMenuItemTagOpenLinkInNewWindow:
-        case WebMenuItemTagOpenImageInNewWindow:
-            if (self.openInNewPageActionVisible) {
-                item.title = QCoreApplication::translate("HelpViewer", "Open Link as New Page").toNSString();
-                [ret addObject:item];
+        case WebMenuItemTagOpenImageInNewWindow: {
+            NSURL *url = [element objectForKey:WebElementLinkURLKey];
+            if (QTC_GUARD(url)) {
+                if (widget->viewer()->isActionVisible(Help::Internal::HelpViewer::Action::NewPage))
+                    [ret addObject:menuItem(url, self, @selector(openAsNewPage:),
+                                            QCoreApplication::translate("HelpViewer", Help::Constants::TR_OPEN_LINK_AS_NEW_PAGE))];
+                if (widget->viewer()->isActionVisible(Help::Internal::HelpViewer::Action::ExternalWindow))
+                    [ret addObject:menuItem(url, self, @selector(openInWindow:),
+                                            QCoreApplication::translate("HelpViewer", Help::Constants::TR_OPEN_LINK_IN_WINDOW))];
             }
+            break;
+        }
         default:
             break;
         }
@@ -329,16 +364,6 @@ static void ensureProtocolHandler()
     return [ret autorelease];
 }
 
-- (WebView *)webView:(WebView *)sender createWebViewWithRequest:(NSURLRequest *)request
-{
-    Q_UNUSED(sender)
-    Q_UNUSED(request)
-    Help::Internal::MacWebKitHelpViewer* viewer
-            = static_cast<Help::Internal::MacWebKitHelpViewer *>(
-                Help::Internal::OpenPagesManager::instance().createPage(QUrl()));
-    return viewer->widget()->webView();
-}
-
 - (void)webView:(WebView *)sender mouseDidMoveOverElement:(NSDictionary *)elementInformation
     modifierFlags:(NSUInteger)modifierFlags
 {
@@ -356,40 +381,14 @@ static void ensureProtocolHandler()
                        QString::fromNSString(url.absoluteString));
 }
 
-@end
-
-// #pragma mark -- MyWebView
-@interface MyWebView : WebView
-@end
-
-// work around Qt + WebView issue QTBUG-26593, that Qt does not pass mouseMoved: events up the event chain,
-// but the Web(HTML)View is only handling mouse moved for hovering etc if the event was passed up
-// to the NSWindow (arguably a bug in Web(HTML)View)
-@implementation MyWebView
-
-- (void)updateTrackingAreas
+- (void)openAsNewPage:(id)item
 {
-    [super updateTrackingAreas];
-    if (NSArray *trackingArray = [self trackingAreas]) {
-        NSUInteger size = [trackingArray count];
-        for (NSUInteger i = 0; i < size; ++i) {
-            NSTrackingArea *t = [trackingArray objectAtIndex:i];
-            [self removeTrackingArea:t];
-        }
-    }
-    NSUInteger trackingOptions = NSTrackingActiveInActiveApp | NSTrackingInVisibleRect
-            | NSTrackingMouseMoved;
-    NSTrackingArea *ta = [[[NSTrackingArea alloc] initWithRect:[self frame]
-                                                      options:trackingOptions
-                                                        owner:self
-                                                     userInfo:nil]
-                                                                autorelease];
-    [self addTrackingArea:ta];
+    widget->viewer()->newPageRequested(QUrl::fromNSURL([item representedObject]));
 }
 
-- (void)mouseMoved:(NSEvent *)theEvent
+- (void)openInWindow:(id)item
 {
-    [self.window mouseMoved:theEvent];
+    widget->viewer()->externalPageRequested(QUrl::fromNSURL([item representedObject]));
 }
 
 @end
@@ -402,8 +401,9 @@ namespace Internal {
 class MacWebKitHelpWidgetPrivate
 {
 public:
-    MacWebKitHelpWidgetPrivate()
-        : m_savedResponder(nil)
+    MacWebKitHelpWidgetPrivate(MacWebKitHelpViewer *parent)
+        : m_viewer(parent),
+          m_savedResponder(nil)
     {
     }
 
@@ -414,6 +414,7 @@ public:
         [m_uiDelegate release];
     }
 
+    MacWebKitHelpViewer *m_viewer;
     WebView *m_webView;
     FrameLoadDelegate *m_frameLoadDelegate;
     UIDelegate *m_uiDelegate;
@@ -426,13 +427,13 @@ public:
 // #pragma mark -- MacWebKitHelpWidget
 
 MacWebKitHelpWidget::MacWebKitHelpWidget(MacWebKitHelpViewer *parent)
-    : QMacCocoaViewContainer(0, parent),
-      d(new MacWebKitHelpWidgetPrivate)
+    : QMacCocoaViewContainer(nullptr, parent),
+      d(new MacWebKitHelpWidgetPrivate(parent))
 {
     d->m_toolTipTimer.setSingleShot(true);
     connect(&d->m_toolTipTimer, &QTimer::timeout, this, &MacWebKitHelpWidget::showToolTip);
     @autoreleasepool {
-        d->m_webView = [[MyWebView alloc] init];
+        d->m_webView = [[WebView alloc] init];
         // Turn layered rendering on.
         // Otherwise the WebView will render empty after any QQuickWidget was shown.
         d->m_webView.wantsLayer = YES;
@@ -444,6 +445,7 @@ MacWebKitHelpWidget::MacWebKitHelpWidget(MacWebKitHelpViewer *parent)
 
         setCocoaView(d->m_webView);
     }
+
 }
 
 MacWebKitHelpWidget::~MacWebKitHelpWidget()
@@ -451,9 +453,9 @@ MacWebKitHelpWidget::~MacWebKitHelpWidget()
     delete d;
 }
 
-void MacWebKitHelpWidget::setOpenInNewPageActionVisible(bool visible)
+MacWebKitHelpViewer *MacWebKitHelpWidget::viewer() const
 {
-    d->m_uiDelegate.openInNewPageActionVisible = visible;
+    return d->m_viewer;
 }
 
 WebView *MacWebKitHelpWidget::webView() const
@@ -463,7 +465,7 @@ WebView *MacWebKitHelpWidget::webView() const
 
 void MacWebKitHelpWidget::startToolTipTimer(const QPoint &pos, const QString &text)
 {
-    int delay = style()->styleHint(QStyle::SH_ToolTip_WakeUpDelay, 0, this, 0);
+    int delay = style()->styleHint(QStyle::SH_ToolTip_WakeUpDelay, nullptr, this, nullptr);
     d->m_toolTipPos = pos;
     d->m_toolTipText = text;
     d->m_toolTipTimer.start(delay);
@@ -492,6 +494,25 @@ void MacWebKitHelpWidget::showToolTip()
 
 // #pragma mark -- MacWebKitHelpViewer
 
+static void responderHack(QWidget *old, QWidget *now)
+{
+    // On focus change, Qt does not make the corresponding QNSView firstResponder.
+    // That breaks when embedding native NSView into a Qt hierarchy. When the focus is changed
+    // by clicking with the mouse into a widget, everything is fine, because Cocoa automatically
+    // adapts firstResponder in that case, but it breaks when setting the Qt focus from code.
+    Q_UNUSED(old)
+    if (!now)
+        return;
+    @autoreleasepool {
+        NSView *view;
+        if (QMacCocoaViewContainer *viewContainer = qobject_cast<QMacCocoaViewContainer *>(now))
+            view = viewContainer->cocoaView();
+        else
+            view = reinterpret_cast<NSView *>(now->effectiveWinId());
+        [view.window makeFirstResponder:view];
+    }
+}
+
 MacWebKitHelpViewer::MacWebKitHelpViewer(QWidget *parent)
     : HelpViewer(parent),
       m_widget(new MacWebKitHelpWidget(this))
@@ -499,7 +520,7 @@ MacWebKitHelpViewer::MacWebKitHelpViewer(QWidget *parent)
     static bool responderHackInstalled = false;
     if (!responderHackInstalled) {
         responderHackInstalled = true;
-        new MacResponderHack(qApp);
+        QObject::connect(qApp, &QApplication::focusChanged, &responderHack);
     }
 
     @autoreleasepool {
@@ -537,21 +558,21 @@ void MacWebKitHelpViewer::setViewerFont(const QFont &font)
 void MacWebKitHelpViewer::scaleUp()
 {
     @autoreleasepool {
-        m_widget->webView().textSizeMultiplier += 0.1;
+        m_widget->webView().textSizeMultiplier += 0.1f;
     }
 }
 
 void MacWebKitHelpViewer::scaleDown()
 {
     @autoreleasepool {
-        m_widget->webView().textSizeMultiplier = qMax(0.1, m_widget->webView().textSizeMultiplier - 0.1);
+        m_widget->webView().textSizeMultiplier = qMax(0.1f, m_widget->webView().textSizeMultiplier - 0.1f);
     }
 }
 
 void MacWebKitHelpViewer::resetScale()
 {
     @autoreleasepool {
-        m_widget->webView().textSizeMultiplier = 1.0;
+        m_widget->webView().textSizeMultiplier = 1.0f;
     }
 }
 
@@ -639,7 +660,7 @@ void MacWebKitHelpViewer::addBackHistoryItems(QMenu *backMenu)
             QAction *action = new QAction(backMenu);
             action->setText(QString::fromNSString([list itemAtIndex:historyIndex].title));
             action->setData(historyIndex);
-            connect(action, SIGNAL(triggered()), this, SLOT(goToHistoryItem()));
+            connect(action, &QAction::triggered, this, &MacWebKitHelpViewer::goToHistoryItem);
             backMenu->addAction(action);
         }
     }
@@ -655,17 +676,12 @@ void MacWebKitHelpViewer::addForwardHistoryItems(QMenu *forwardMenu)
             QAction *action = new QAction(forwardMenu);
             action->setText(QString::fromNSString([list itemAtIndex:historyIndex].title));
             action->setData(historyIndex);
-            connect(action, SIGNAL(triggered()), this, SLOT(goToHistoryItem()));
+            connect(action, &QAction::triggered, this, &MacWebKitHelpViewer::goToHistoryItem);
             forwardMenu->addAction(action);
         }
     }
 }
 
-void MacWebKitHelpViewer::setOpenInNewPageActionVisible(bool visible)
-{
-    m_widget->setOpenInNewPageActionVisible(visible);
-}
-
 DOMRange *MacWebKitHelpViewer::findText(NSString *text, bool forward, bool caseSensitive, DOMNode *startNode, int startOffset)
 {
     QTC_ASSERT(text, return nil);
@@ -855,33 +871,5 @@ void MacWebKitHelpViewer::goToHistoryItem()
     }
 }
 
-// #pragma mark -- MacResponderHack
-
-MacResponderHack::MacResponderHack(QObject *parent)
-    : QObject(parent)
-{
-    connect(qApp, SIGNAL(focusChanged(QWidget*,QWidget*)),
-            this, SLOT(responderHack(QWidget*,QWidget*)));
-}
-
-void MacResponderHack::responderHack(QWidget *old, QWidget *now)
-{
-    // On focus change, Qt does not make the corresponding QNSView firstResponder.
-    // That breaks when embedding native NSView into a Qt hierarchy. When the focus is changed
-    // by clicking with the mouse into a widget, everything is fine, because Cocoa automatically
-    // adapts firstResponder in that case, but it breaks when setting the Qt focus from code.
-    Q_UNUSED(old)
-    if (!now)
-        return;
-    @autoreleasepool {
-        NSView *view;
-        if (QMacCocoaViewContainer *viewContainer = qobject_cast<QMacCocoaViewContainer *>(now))
-            view = viewContainer->cocoaView();
-        else
-            view = (NSView *)now->effectiveWinId();
-        [view.window makeFirstResponder:view];
-    }
-}
-
 } // Internal
 } // Help
