@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,39 +9,46 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "navigatorview.h"
 #include "navigatortreemodel.h"
 #include "navigatorwidget.h"
+#include "qmldesignerconstants.h"
+#include "qmldesignericons.h"
+
 #include "nameitemdelegate.h"
 #include "iconcheckboxitemdelegate.h"
 
+#include <bindingproperty.h>
+#include <designmodecontext.h>
+#include <designersettings.h>
+#include <nodeproperty.h>
+#include <nodelistproperty.h>
+#include <variantproperty.h>
+#include <qmlitemnode.h>
+#include <rewritingexception.h>
+
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/icore.h>
 
-#include <designmodecontext.h>
-#include <nodeproperty.h>
-#include <nodelistproperty.h>
-#include <variantproperty.h>
+#include <utils/icon.h>
+#include <utils/utilsicons.h>
+
 #include <QHeaderView>
-#include <qmlitemnode.h>
+
 
 static inline void setScenePos(const QmlDesigner::ModelNode &modelNode,const QPointF &pos)
 {
@@ -62,46 +69,58 @@ static inline void setScenePos(const QmlDesigner::ModelNode &modelNode,const QPo
 namespace QmlDesigner {
 
 NavigatorView::NavigatorView(QObject* parent) :
-        AbstractView(parent),
-        m_blockSelectionChangedSignal(false),
-        m_widget(new NavigatorWidget(this)),
-        m_treeModel(new NavigatorTreeModel(this))
+    AbstractView(parent),
+    m_blockSelectionChangedSignal(false),
+    m_widget(new NavigatorWidget(this)),
+    m_treeModel(new NavigatorTreeModel(this))
 {
+#ifndef QMLDESIGNER_TEST
     Internal::NavigatorContext *navigatorContext = new Internal::NavigatorContext(m_widget.data());
     Core::ICore::addContextObject(navigatorContext);
+#endif
 
+    m_treeModel->setView(this);
     m_widget->setTreeModel(m_treeModel.data());
+    m_currentModelInterface = m_treeModel;
 
-    connect(treeWidget()->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)), this, SLOT(changeSelection(QItemSelection,QItemSelection)));
+    connect(treeWidget()->selectionModel(), &QItemSelectionModel::selectionChanged, this, &NavigatorView::changeSelection);
 
-    connect(m_widget.data(), SIGNAL(leftButtonClicked()), this, SLOT(leftButtonClicked()));
-    connect(m_widget.data(), SIGNAL(rightButtonClicked()), this, SLOT(rightButtonClicked()));
-    connect(m_widget.data(), SIGNAL(downButtonClicked()), this, SLOT(downButtonClicked()));
-    connect(m_widget.data(), SIGNAL(upButtonClicked()), this, SLOT(upButtonClicked()));
+    connect(m_widget.data(), &NavigatorWidget::leftButtonClicked, this, &NavigatorView::leftButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::rightButtonClicked, this, &NavigatorView::rightButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::downButtonClicked, this, &NavigatorView::downButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::upButtonClicked, this, &NavigatorView::upButtonClicked);
+    connect(m_widget.data(), &NavigatorWidget::filterToggled, this, &NavigatorView::filterToggled);
 
-    treeWidget()->setIndentation(treeWidget()->indentation() * 0.5);
+#ifndef QMLDESIGNER_TEST
+    NameItemDelegate *idDelegate = new NameItemDelegate(this);
+    IconCheckboxItemDelegate *showDelegate =
+            new IconCheckboxItemDelegate(this,
+                                         Utils::Icons::EYE_OPEN_TOOLBAR.icon(),
+                                         Utils::Icons::EYE_CLOSED_TOOLBAR.icon());
 
-    NameItemDelegate *idDelegate = new NameItemDelegate(this,
-                                                        m_treeModel.data());
-    IconCheckboxItemDelegate *showDelegate = new IconCheckboxItemDelegate(this,
-                                                                          ":/qmldesigner/images/eye_open.png",
-                                                                          ":/qmldesigner/images/placeholder.png",
-                                                                          m_treeModel.data());
+    IconCheckboxItemDelegate *exportDelegate =
+            new IconCheckboxItemDelegate(this,
+                                         Icons::EXPORT_CHECKED.icon(),
+                                         Icons::EXPORT_UNCHECKED.icon());
 
 #ifdef _LOCK_ITEMS_
-    IconCheckboxItemDelegate *lockDelegate = new IconCheckboxItemDelegate(this,":/qmldesigner/images/lock.png",
-                                                          ":/qmldesigner/images/hole.png",m_treeModel.data());
+    IconCheckboxItemDelegate *lockDelegate =
+            new IconCheckboxItemDelegate(this,
+                                         Utils::Icons::LOCKED_TOOLBAR.icon(),
+                                         Utils::Icons::UNLOCKED_TOOLBAR.icon());
 #endif
 
 
-    treeWidget()->setItemDelegateForColumn(0,idDelegate);
+    treeWidget()->setItemDelegateForColumn(0, idDelegate);
 #ifdef _LOCK_ITEMS_
     treeWidget()->setItemDelegateForColumn(1,lockDelegate);
     treeWidget()->setItemDelegateForColumn(2,showDelegate);
 #else
-    treeWidget()->setItemDelegateForColumn(1,showDelegate);
+    treeWidget()->setItemDelegateForColumn(1, exportDelegate);
+    treeWidget()->setItemDelegateForColumn(2, showDelegate);
 #endif
 
+#endif //QMLDESIGNER_TEST
 }
 
 NavigatorView::~NavigatorView()
@@ -128,14 +147,14 @@ void NavigatorView::modelAttached(Model *model)
 {
     AbstractView::modelAttached(model);
 
-    m_treeModel->setView(this);
+    m_currentModelInterface->setFilter(
+                DesignerSettings::getValue(DesignerSettingsKey::NAVIGATOR_SHOW_ONLY_VISIBLE_ITEMS).toBool());
 
     QTreeView *treeView = treeWidget();
     treeView->expandAll();
 
-    treeView->header()->setResizeMode(0, QHeaderView::Stretch);
+    treeView->header()->setSectionResizeMode(0, QHeaderView::Stretch);
     treeView->header()->resizeSection(1,26);
-    treeView->setRootIsDecorated(false);
     treeView->setIndentation(20);
 #ifdef _LOCK_ITEMS_
     treeView->header()->resizeSection(2,20);
@@ -144,8 +163,6 @@ void NavigatorView::modelAttached(Model *model)
 
 void NavigatorView::modelAboutToBeDetached(Model *model)
 {
-    m_treeModel->removeSubTree(rootModelNode());
-    m_treeModel->clearView();
     AbstractView::modelAboutToBeDetached(model);
 }
 
@@ -154,48 +171,136 @@ void NavigatorView::importsChanged(const QList<Import> &/*addedImports*/, const
     treeWidget()->update();
 }
 
-void NavigatorView::nodeCreated(const ModelNode & /*createdNode*/)
+void NavigatorView::bindingPropertiesChanged(const QList<BindingProperty> & propertyList, PropertyChangeFlags /*propertyChange*/)
+{
+    for (const BindingProperty &bindingProperty : propertyList) {
+        /* If a binding property that exports an item using an alias property has
+         * changed, we have to update the affected item.
+         */
+
+        if (bindingProperty.isAliasExport())
+            m_currentModelInterface->notifyDataChanged(modelNodeForId(bindingProperty.expression()));
+    }
+}
+
+void NavigatorView::handleChangedExport(const ModelNode &modelNode, bool exported)
+{
+    const ModelNode rootNode = rootModelNode();
+    Q_ASSERT(rootNode.isValid());
+    const PropertyName modelNodeId = modelNode.id().toUtf8();
+    if (rootNode.hasProperty(modelNodeId))
+        rootNode.removeProperty(modelNodeId);
+    if (exported) {
+        try {
+            RewriterTransaction transaction =
+                    beginRewriterTransaction(QByteArrayLiteral("NavigatorTreeModel:exportItem"));
+
+            QmlObjectNode qmlObjectNode(modelNode);
+            qmlObjectNode.ensureAliasExport();
+            transaction.commit();
+        }  catch (RewritingException &exception) { //better safe than sorry! There always might be cases where we fail
+            exception.showException();
+        }
+    }
+}
+
+bool NavigatorView::isNodeInvisible(const ModelNode &modelNode) const
+{
+    return modelNode.auxiliaryData("invisible").toBool();
+}
+
+void NavigatorView::disableWidget()
+{
+    if (m_widget)
+        m_widget->disableNavigator();
+}
+
+void NavigatorView::enableWidget()
+{
+    if (m_widget)
+        m_widget->enableNavigator();
+}
+
+ModelNode NavigatorView::modelNodeForIndex(const QModelIndex &modelIndex) const
+{
+    return modelIndex.model()->data(modelIndex, ModelNodeRole).value<ModelNode>();
+}
+
+void NavigatorView::nodeAboutToBeRemoved(const ModelNode & /*removedNode*/)
 {
 }
 
-void NavigatorView::nodeRemoved(const ModelNode & /*removedNode*/, const NodeAbstractProperty & /*parentProperty*/, PropertyChangeFlags /*propertyChange*/)
+void NavigatorView::nodeRemoved(const ModelNode &removedNode,
+                                const NodeAbstractProperty & /*parentProperty*/,
+                                AbstractView::PropertyChangeFlags /*propertyChange*/)
+{
+    m_currentModelInterface->notifyModelNodesRemoved({removedNode});
+}
+
+void NavigatorView::nodeReparented(const ModelNode &modelNode,
+                                   const NodeAbstractProperty & /*newPropertyParent*/,
+                                   const NodeAbstractProperty & oldPropertyParent,
+                                   AbstractView::PropertyChangeFlags /*propertyChange*/)
+{
+    if (!oldPropertyParent.isValid())
+        m_currentModelInterface->notifyModelNodesInserted({modelNode});
+    else
+        m_currentModelInterface->notifyModelNodesMoved({modelNode});
+    treeWidget()->expand(indexForModelNode(modelNode));
+}
+
+void NavigatorView::nodeIdChanged(const ModelNode& modelNode, const QString & /*newId*/, const QString & /*oldId*/)
+{
+    m_currentModelInterface->notifyDataChanged(modelNode);
+}
+
+void NavigatorView::propertiesAboutToBeRemoved(const QList<AbstractProperty>& /*propertyList*/)
 {
 }
 
-void NavigatorView::propertiesRemoved(const QList<AbstractProperty> & /*propertyList*/)
+void NavigatorView::propertiesRemoved(const QList<AbstractProperty> &propertyList)
 {
+    QList<ModelNode> modelNodes;
+    for (const AbstractProperty &property : propertyList) {
+        if (property.isNodeAbstractProperty()) {
+            NodeAbstractProperty nodeAbstractProperty(property.toNodeListProperty());
+            modelNodes.append(nodeAbstractProperty.directSubNodes());
+        }
+    }
+
+    m_currentModelInterface->notifyModelNodesRemoved(modelNodes);
 }
 
-void NavigatorView::variantPropertiesChanged(const QList<VariantProperty> & /*propertyList*/, PropertyChangeFlags /*propertyChange*/)
+void NavigatorView::rootNodeTypeChanged(const QString & /*type*/, int /*majorVersion*/, int /*minorVersion*/)
 {
+    m_currentModelInterface->notifyDataChanged(rootModelNode());
 }
 
-void NavigatorView::bindingPropertiesChanged(const QList<BindingProperty> & /*propertyList*/, PropertyChangeFlags /*propertyChange*/)
+void NavigatorView::nodeTypeChanged(const ModelNode &modelNode, const TypeName &, int , int)
 {
+    m_currentModelInterface->notifyDataChanged(modelNode);
 }
 
-void NavigatorView::signalHandlerPropertiesChanged(const QVector<SignalHandlerProperty> & /*propertyList*/,
-                                                   AbstractView::PropertyChangeFlags /*propertyChange*/)
+void NavigatorView::auxiliaryDataChanged(const ModelNode &modelNode,
+                                         const PropertyName & /*name*/,
+                                         const QVariant & /*data*/)
 {
+    m_currentModelInterface->notifyDataChanged(modelNode);
 }
 
-void NavigatorView::nodeAboutToBeRemoved(const ModelNode &removedNode)
+void NavigatorView::instanceErrorChanged(const QVector<ModelNode> &errorNodeList)
 {
-    m_treeModel->removeSubTree(removedNode);
+    foreach (const ModelNode &modelNode, errorNodeList)
+        m_currentModelInterface->notifyDataChanged(modelNode);
 }
 
-void NavigatorView::nodeAboutToBeReparented(const ModelNode &/*node*/, const NodeAbstractProperty &/*newPropertyParent*/, const NodeAbstractProperty &/*oldPropertyParent*/, AbstractView::PropertyChangeFlags /*propertyChange*/)
-{
-}
-
-void NavigatorView::nodeReparented(const ModelNode &node, const NodeAbstractProperty & /*newPropertyParent*/, const NodeAbstractProperty & /*oldPropertyParent*/, AbstractView::PropertyChangeFlags /*propertyChange*/)
+void NavigatorView::nodeOrderChanged(const NodeListProperty & listProperty,
+                                     const ModelNode & /*node*/,
+                                     int /*oldIndex*/)
 {
     bool blocked = blockSelectionChangedSignal(true);
 
-    m_treeModel->removeSubTree(node);
-
-    if (node.isInHierarchy())
-        m_treeModel->addSubTree(node);
+    m_currentModelInterface->notifyModelNodesMoved(listProperty.directSubNodes());
 
     // make sure selection is in sync again
     updateItemSelection();
@@ -203,118 +308,26 @@ void NavigatorView::nodeReparented(const ModelNode &node, const NodeAbstractProp
     blockSelectionChangedSignal(blocked);
 }
 
-void NavigatorView::nodeIdChanged(const ModelNode& node, const QString & /*newId*/, const QString & /*oldId*/)
-{
-    if (m_treeModel->isInTree(node))
-        m_treeModel->updateItemRow(node);
-}
-
-void NavigatorView::propertiesAboutToBeRemoved(const QList<AbstractProperty>& propertyList)
-{
-    foreach (const AbstractProperty &property, propertyList) {
-        if (property.isNodeAbstractProperty()) {
-            NodeAbstractProperty nodeAbstractProperty(property.toNodeListProperty());
-            foreach (const ModelNode &childNode, nodeAbstractProperty.directSubNodes()) {
-                m_treeModel->removeSubTree(childNode);
-            }
-        }
-    }
-}
-
-void NavigatorView::rootNodeTypeChanged(const QString & /*type*/, int /*majorVersion*/, int /*minorVersion*/)
-{
-    if (m_treeModel->isInTree(rootModelNode()))
-        m_treeModel->updateItemRow(rootModelNode());
-}
-
-void NavigatorView::auxiliaryDataChanged(const ModelNode &modelNode, const PropertyName & name, const QVariant & /*data*/)
-{
-    if (name == "invisible" && m_treeModel->isInTree(modelNode))
-    {
-        // update model
-        m_treeModel->updateItemRow(modelNode);
-
-        // repaint row (id and icon)
-        foreach (const ModelNode &currentModelNode, modelNode.allSubModelNodesAndThisNode()) {
-            QModelIndex index = m_treeModel->indexForNode(currentModelNode);
-            treeWidget()->update(index);
-            treeWidget()->update(index.sibling(index.row(),index.column()+1));
-        }
-    }
-}
-
-void NavigatorView::scriptFunctionsChanged(const ModelNode &/*node*/, const QStringList &/*scriptFunctionList*/)
-{
-}
-
-void NavigatorView::instancePropertyChange(const QList<QPair<ModelNode, PropertyName> > &/*propertyList*/)
-{
-}
-
-void NavigatorView::instancesCompleted(const QVector<ModelNode> &/*completedNodeList*/)
-{
-}
-
-void NavigatorView::instanceInformationsChange(const QMultiHash<ModelNode, InformationName> &/*informationChangeHash*/)
-{
-}
-
-void NavigatorView::instancesRenderImageChanged(const QVector<ModelNode> &/*nodeList*/)
-{
-}
-
-void NavigatorView::instancesPreviewImageChanged(const QVector<ModelNode> &/*nodeList*/)
-{
-}
-
-void NavigatorView::instancesChildrenChanged(const QVector<ModelNode> &/*nodeList*/)
-{
-
-}
-
-void NavigatorView::nodeSourceChanged(const ModelNode &, const QString & /*newNodeSource*/)
-{
-
-}
-
-void NavigatorView::rewriterBeginTransaction()
-{
-}
-
-void NavigatorView::rewriterEndTransaction()
-{
-}
-
-void NavigatorView::currentStateChanged(const ModelNode &/*node*/)
-{
-}
-
-void NavigatorView::instancesToken(const QString &/*tokenName*/, int /*tokenNumber*/, const QVector<ModelNode> &/*nodeVector*/)
-{
-
-}
-
-void NavigatorView::nodeOrderChanged(const NodeListProperty &listProperty, const ModelNode &node, int oldIndex)
-{
-    if (m_treeModel->isInTree(node)) {
-        m_treeModel->removeSubTree(listProperty.parentModelNode());
-
-        if (node.isInHierarchy())
-            m_treeModel->addSubTree(listProperty.parentModelNode());
-
-    }
-}
-
 void NavigatorView::changeToComponent(const QModelIndex &index)
 {
-    if (index.isValid() && m_treeModel->data(index, Qt::UserRole).isValid()) {
-        ModelNode doubleClickNode = m_treeModel->nodeForIndex(index);
+    if (index.isValid() && currentModel()->data(index, Qt::UserRole).isValid()) {
+        const ModelNode doubleClickNode = modelNodeForIndex(index);
         if (doubleClickNode.metaInfo().isFileComponent())
             Core::EditorManager::openEditor(doubleClickNode.metaInfo().componentFileName(),
                                             Core::Id(), Core::EditorManager::DoNotMakeVisible);
     }
 }
 
+QModelIndex NavigatorView::indexForModelNode(const ModelNode &modelNode) const
+{
+    return m_currentModelInterface->indexForModelNode(modelNode);
+}
+
+QAbstractItemModel *NavigatorView::currentModel() const
+{
+    return treeWidget()->model();
+}
+
 void NavigatorView::leftButtonClicked()
 {
     if (selectedModelNodes().count() > 1)
@@ -326,14 +339,15 @@ void NavigatorView::leftButtonClicked()
         if (!node.isRootNode() && !node.parentProperty().parentModelNode().isRootNode()) {
             if (QmlItemNode::isValidQmlItemNode(node)) {
                 QPointF scenePos = QmlItemNode(node).instanceScenePosition();
-                node.parentProperty().parentProperty().reparentHere(node);
+                reparentAndCatch(node.parentProperty().parentProperty(), node);
                 if (!scenePos.isNull())
                     setScenePos(node, scenePos);
             } else {
-                node.parentProperty().parentProperty().reparentHere(node);
+                reparentAndCatch(node.parentProperty().parentProperty(), node);
             }
         }
     }
+
     updateItemSelection();
     blockSelectionChangedSignal(blocked);
 }
@@ -355,12 +369,12 @@ void NavigatorView::rightButtonClicked()
                         && QmlItemNode::isValidQmlItemNode(newParent)
                         && !newParent.metaInfo().defaultPropertyIsComponent()) {
                     QPointF scenePos = QmlItemNode(node).instanceScenePosition();
-                    newParent.nodeAbstractProperty(newParent.metaInfo().defaultPropertyName()).reparentHere(node);
+                    reparentAndCatch(newParent.nodeAbstractProperty(newParent.metaInfo().defaultPropertyName()), node);
                     if (!scenePos.isNull())
                         setScenePos(node, scenePos);
                 } else {
                     if (newParent.metaInfo().isValid() && !newParent.metaInfo().defaultPropertyIsComponent())
-                        newParent.nodeAbstractProperty(newParent.metaInfo().defaultPropertyName()).reparentHere(node);
+                        reparentAndCatch(newParent.nodeAbstractProperty(newParent.metaInfo().defaultPropertyName()), node);
                 }
             }
         }
@@ -403,14 +417,25 @@ void NavigatorView::downButtonClicked()
     blockSelectionChangedSignal(blocked);
 }
 
+void NavigatorView::filterToggled(bool flag)
+{
+    m_currentModelInterface->setFilter(flag);
+    treeWidget()->expandAll();
+    DesignerSettings::setValue(DesignerSettingsKey::NAVIGATOR_SHOW_ONLY_VISIBLE_ITEMS, flag);
+}
+
 void NavigatorView::changeSelection(const QItemSelection & /*newSelection*/, const QItemSelection &/*deselected*/)
 {
     if (m_blockSelectionChangedSignal)
         return;
+
     QSet<ModelNode> nodeSet;
-    foreach (const QModelIndex &index, treeWidget()->selectionModel()->selectedIndexes()) {
-        if (m_treeModel->data(index, Qt::UserRole).isValid())
-            nodeSet.insert(m_treeModel->nodeForIndex(index));
+
+    for (const QModelIndex &index : treeWidget()->selectionModel()->selectedIndexes()) {
+
+        const ModelNode modelNode = modelNodeForIndex(index);
+        if (modelNode.isValid())
+            nodeSet.insert(modelNode);
     }
 
     bool blocked = blockSelectionChangedSignal(true);
@@ -427,10 +452,10 @@ void NavigatorView::updateItemSelection()
 {
     QItemSelection itemSelection;
     foreach (const ModelNode &node, selectedModelNodes()) {
-        const QModelIndex index = m_treeModel->indexForNode(node);
+        const QModelIndex index = indexForModelNode(node);
         if (index.isValid()) {
-            const QModelIndex beginIndex(m_treeModel->index(index.row(), 0, index.parent()));
-            const QModelIndex endIndex(m_treeModel->index(index.row(), m_treeModel->columnCount(index.parent()) - 1, index.parent()));
+            const QModelIndex beginIndex(currentModel()->index(index.row(), 0, index.parent()));
+            const QModelIndex endIndex(currentModel()->index(index.row(), currentModel()->columnCount(index.parent()) - 1, index.parent()));
             if (beginIndex.isValid() && endIndex.isValid())
                 itemSelection.select(beginIndex, endIndex);
         }
@@ -441,7 +466,7 @@ void NavigatorView::updateItemSelection()
     blockSelectionChangedSignal(blocked);
 
     if (!selectedModelNodes().isEmpty())
-        treeWidget()->scrollTo(m_treeModel->indexForNode(selectedModelNodes().first()));
+        treeWidget()->scrollTo(indexForModelNode(selectedModelNodes().constFirst()));
 
     // make sure selected nodes a visible
     foreach (const QModelIndex &selectedIndex, itemSelection.indexes()) {
@@ -450,7 +475,7 @@ void NavigatorView::updateItemSelection()
     }
 }
 
-QTreeView *NavigatorView::treeWidget()
+QTreeView *NavigatorView::treeWidget() const
 {
     if (m_widget)
         return m_widget->treeView();
@@ -480,4 +505,13 @@ void NavigatorView::expandRecursively(const QModelIndex &index)
     }
 }
 
+void NavigatorView::reparentAndCatch(NodeAbstractProperty property, const ModelNode &modelNode)
+{
+    try {
+        property.reparentHere(modelNode);
+    }  catch (Exception &exception) {
+        exception.showException();
+    }
+}
+
 } // namespace QmlDesigner
