@@ -39,9 +39,9 @@
 #include <qmljs/qmljspropertyreader.h>
 #include <qmljs/qmljsrewriter.h>
 #include <qmljstools/qmljsrefactoringchanges.h>
-#include <projectexplorer/session.h>
-#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/projectnodes.h>
+#include <projectexplorer/projecttree.h>
 
 #include <utils/fileutils.h>
 
@@ -111,6 +111,8 @@ public:
         QStringList result;
         QStringList sourcePreview;
 
+        QString suffix;
+
         if (!m_idName.isEmpty())
             sourcePreview.append(QLatin1String("    id: ") + m_idName);
         else
@@ -127,7 +129,7 @@ public:
         foreach (const QString &property, sortedPropertiesWithoutId)
             sourcePreview.append(QLatin1String("    ") + property + QLatin1String(": ") + propertyReader.readAstValue(property));
 
-        bool confirm = ComponentNameDialog::go(&componentName, &path,
+        const bool confirm = ComponentNameDialog::go(&componentName, &path, &suffix,
                                                sortedPropertiesWithoutId,
                                                sourcePreview,
                                                QFileInfo(currentFileName).fileName(),
@@ -140,7 +142,7 @@ public:
             return;
 
         const QString newFileName = path + QLatin1Char('/') + componentName
-                + QLatin1String(".qml");
+                + QLatin1String(".") + suffix;
 
         QString imports;
         UiProgram *prog = currentFile->qmljsDocument()->qmlProgram();
@@ -190,7 +192,7 @@ public:
         if (path == QFileInfo(currentFileName).path()) {
             // hack for the common case, next version should use the wizard
             ProjectExplorer::Node * oldFileNode =
-                    ProjectExplorer::SessionManager::nodeForFile(Utils::FileName::fromString(currentFileName));
+                    ProjectExplorer::ProjectTree::nodeForFile(Utils::FileName::fromString(currentFileName));
             if (oldFileNode) {
                 ProjectExplorer::FolderNode *containingFolder = oldFileNode->parentFolderNode();
                 if (containingFolder)
@@ -231,7 +233,7 @@ public:
 } // end of anonymous namespace
 
 
-void ComponentFromObjectDef::match(const QmlJSQuickFixInterface &interface, QuickFixOperations &result)
+void matchComponentFromObjectDefQuickFix(const QmlJSQuickFixInterface &interface, QuickFixOperations &result)
 {
     const int pos = interface->currentFile()->cursor().position();
 
@@ -244,19 +246,19 @@ void ComponentFromObjectDef::match(const QmlJSQuickFixInterface &interface, Quic
                 return;
              // check that the node is not the root node
             if (i > 0 && !cast<UiProgram*>(path.at(i - 1))) {
-                result.append(new Operation(interface, objDef));
+                result << new Operation(interface, objDef);
                 return;
             }
         } else if (UiObjectBinding *objBinding = cast<UiObjectBinding *>(node)) {
             if (!interface->currentFile()->isCursorOn(objBinding->qualifiedTypeNameId))
                 return;
-            result.append(new Operation(interface, objBinding));
+            result << new Operation(interface, objBinding);
             return;
         }
     }
 }
 
-void ComponentFromObjectDef::perform(const QString &fileName, QmlJS::AST::UiObjectDefinition *objDef)
+void performComponentFromObjectDef(const QString &fileName, QmlJS::AST::UiObjectDefinition *objDef)
 {
     QmlJSRefactoringChanges refactoring(QmlJS::ModelManagerInterface::instance(),
                                         QmlJS::ModelManagerInterface::instance()->snapshot());
