@@ -34,6 +34,10 @@
 using namespace TextEditor;
 using namespace TextEditor::Internal;
 
+namespace {
+const int layoutSpacing = 6;
+} // namespace
+
 static inline QString colorButtonStyleSheet(const QColor &bgColor)
 {
     if (bgColor.isValid()) {
@@ -145,12 +149,10 @@ private:
 
 ColorSchemeEdit::ColorSchemeEdit(QWidget *parent) :
     QWidget(parent),
-    m_curItem(-1),
     m_ui(new Ui::ColorSchemeEdit),
-    m_formatsModel(new FormatsModel(this)),
-    m_readOnly(false)
+    m_formatsModel(new FormatsModel(this))
 {
-    setPalette(Utils::Theme::initialPalette());
+    setContentsMargins(0, layoutSpacing, 0, 0);
     m_ui->setupUi(this);
     m_ui->itemList->setModel(m_formatsModel);
 
@@ -166,6 +168,14 @@ ColorSchemeEdit::ColorSchemeEdit(QWidget *parent) :
             this, &ColorSchemeEdit::eraseBackColor);
     connect(m_ui->eraseForegroundToolButton, &QAbstractButton::clicked,
             this, &ColorSchemeEdit::eraseForeColor);
+    connect(m_ui->foregroundSaturationSpinBox, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
+            this, &ColorSchemeEdit::changeRelativeForeColor);
+    connect(m_ui->foregroundLightnessSpinBox, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
+            this, &ColorSchemeEdit::changeRelativeForeColor);
+    connect(m_ui->backgroundSaturationSpinBox, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
+            this, &ColorSchemeEdit::changeRelativeBackColor);
+    connect(m_ui->backgroundLightnessSpinBox, static_cast<void (QDoubleSpinBox::*)(double)>(&QDoubleSpinBox::valueChanged),
+            this, &ColorSchemeEdit::changeRelativeBackColor);
     connect(m_ui->boldCheckBox, &QAbstractButton::toggled,
             this, &ColorSchemeEdit::checkCheckBoxes);
     connect(m_ui->italicCheckBox, &QAbstractButton::toggled,
@@ -211,8 +221,21 @@ void ColorSchemeEdit::setReadOnly(bool readOnly)
     m_ui->backgroundToolButton->setEnabled(enabled);
     m_ui->eraseBackgroundToolButton->setEnabled(enabled);
     m_ui->eraseForegroundToolButton->setEnabled(enabled);
+    m_ui->relativeForegroundHeadline->setEnabled(enabled);
+    m_ui->foregroundSaturationLabel->setEnabled(enabled);
+    m_ui->foregroundLightnessLabel->setEnabled(enabled);
+    m_ui->foregroundSaturationSpinBox->setEnabled(enabled);
+    m_ui->foregroundLightnessSpinBox->setEnabled(enabled);
+    m_ui->relativeBackgroundHeadline->setEnabled(enabled);
+    m_ui->backgroundSaturationLabel->setEnabled(enabled);
+    m_ui->backgroundLightnessLabel->setEnabled(enabled);
+    m_ui->backgroundSaturationSpinBox->setEnabled(enabled);
+    m_ui->backgroundLightnessSpinBox->setEnabled(enabled);
+    m_ui->fontHeadline->setEnabled(enabled);
     m_ui->boldCheckBox->setEnabled(enabled);
     m_ui->italicCheckBox->setEnabled(enabled);
+    m_ui->underlineHeadline->setEnabled(enabled);
+    m_ui->underlineLabel->setEnabled(enabled);
     m_ui->underlineColorToolButton->setEnabled(enabled);
     m_ui->eraseUnderlineColorToolButton->setEnabled(enabled);
     m_ui->underlineComboBox->setEnabled(enabled);
@@ -244,6 +267,8 @@ void ColorSchemeEdit::updateControls()
 {
     updateForegroundControls();
     updateBackgroundControls();
+    updateRelativeForegroundControls();
+    updateRelativeBackgroundControls();
     updateFontControls();
     updateUnderlineControls();
 }
@@ -253,11 +278,12 @@ void ColorSchemeEdit::updateForegroundControls()
     const auto &formatDescription = m_descriptions[m_curItem];
     const Format &format = m_scheme.formatFor(formatDescription.id());
 
-    bool isVisble = formatDescription.showControl(FormatDescription::ShowForegroundControl);
+    bool isVisible = formatDescription.showControl(FormatDescription::ShowForegroundControl);
 
-    m_ui->foregroundLabel->setVisible(isVisble);
-    m_ui->foregroundToolButton->setVisible(isVisble);
-    m_ui->eraseForegroundToolButton->setVisible(isVisble);
+    m_ui->foregroundLabel->setVisible(isVisible);
+    m_ui->foregroundToolButton->setVisible(isVisible);
+    m_ui->eraseForegroundToolButton->setVisible(isVisible);
+    m_ui->foregroundSpacer->setVisible(isVisible);
 
     m_ui->foregroundToolButton->setStyleSheet(colorButtonStyleSheet(format.foreground()));
     m_ui->eraseForegroundToolButton->setEnabled(!m_readOnly
@@ -270,11 +296,12 @@ void ColorSchemeEdit::updateBackgroundControls()
     const auto formatDescription = m_descriptions[m_curItem];
     const Format &format = m_scheme.formatFor(formatDescription.id());
 
-    bool isVisble = formatDescription.showControl(FormatDescription::ShowBackgroundControl);
+    bool isVisible = formatDescription.showControl(FormatDescription::ShowBackgroundControl);
 
-    m_ui->backgroundLabel->setVisible(isVisble);
-    m_ui->backgroundToolButton->setVisible(isVisble);
-    m_ui->eraseBackgroundToolButton->setVisible(isVisble);
+    m_ui->backgroundLabel->setVisible(isVisible);
+    m_ui->backgroundToolButton->setVisible(isVisible);
+    m_ui->eraseBackgroundToolButton->setVisible(isVisible);
+    m_ui->backgroundSpacer->setVisible(isVisible);
 
     m_ui->backgroundToolButton->setStyleSheet(colorButtonStyleSheet(format.background()));
     m_ui->eraseBackgroundToolButton->setEnabled(!m_readOnly
@@ -282,6 +309,52 @@ void ColorSchemeEdit::updateBackgroundControls()
                                                 && format.background().isValid());
 }
 
+void ColorSchemeEdit::updateRelativeForegroundControls()
+{
+    const auto &formatDescription = m_descriptions[m_curItem];
+    const Format &format = m_scheme.formatFor(formatDescription.id());
+
+    QSignalBlocker saturationSignalBlocker(m_ui->foregroundSaturationSpinBox);
+    QSignalBlocker lightnessSignalBlocker(m_ui->foregroundLightnessSpinBox);
+
+    bool isVisible = formatDescription.showControl(FormatDescription::ShowRelativeForegroundControl);
+
+    m_ui->relativeForegroundHeadline->setVisible(isVisible);
+    m_ui->foregroundSaturationLabel->setVisible(isVisible);
+    m_ui->foregroundLightnessLabel->setVisible(isVisible);
+    m_ui->foregroundSaturationSpinBox->setVisible(isVisible);
+    m_ui->foregroundLightnessSpinBox->setVisible(isVisible);
+    m_ui->relativeForegroundSpacer1->setVisible(isVisible);
+    m_ui->relativeForegroundSpacer2->setVisible(isVisible);
+    m_ui->relativeForegroundSpacer3->setVisible(isVisible);
+
+    m_ui->foregroundSaturationSpinBox->setValue(format.relativeForegroundSaturation());
+    m_ui->foregroundLightnessSpinBox->setValue(format.relativeForegroundLightness());
+}
+
+void ColorSchemeEdit::updateRelativeBackgroundControls()
+{
+    const auto &formatDescription = m_descriptions[m_curItem];
+    const Format &format = m_scheme.formatFor(formatDescription.id());
+
+    QSignalBlocker saturationSignalBlocker(m_ui->backgroundSaturationSpinBox);
+    QSignalBlocker lightnessSignalBlocker(m_ui->backgroundLightnessSpinBox);
+
+    bool isVisible = formatDescription.showControl(FormatDescription::ShowRelativeBackgroundControl);
+
+    m_ui->relativeBackgroundHeadline->setVisible(isVisible);
+    m_ui->backgroundSaturationLabel->setVisible(isVisible);
+    m_ui->backgroundLightnessLabel->setVisible(isVisible);
+    m_ui->backgroundSaturationSpinBox->setVisible(isVisible);
+    m_ui->backgroundLightnessSpinBox->setVisible(isVisible);
+    m_ui->relativeBackgroundSpacer1->setVisible(isVisible);
+    m_ui->relativeBackgroundSpacer2->setVisible(isVisible);
+    m_ui->relativeBackgroundSpacer3->setVisible(isVisible);
+
+    m_ui->backgroundSaturationSpinBox->setValue(format.relativeBackgroundSaturation());
+    m_ui->backgroundLightnessSpinBox->setValue(format.relativeBackgroundLightness());
+}
+
 void ColorSchemeEdit::updateFontControls()
 {
     const auto formatDescription = m_descriptions[m_curItem];
@@ -290,10 +363,13 @@ void ColorSchemeEdit::updateFontControls()
     QSignalBlocker boldSignalBlocker(m_ui->boldCheckBox);
     QSignalBlocker italicSignalBlocker(m_ui->italicCheckBox);
 
-    bool isVisble= formatDescription.showControl(FormatDescription::ShowFontControls);
+    bool isVisible= formatDescription.showControl(FormatDescription::ShowFontControls);
 
-    m_ui->boldCheckBox->setVisible(isVisble);
-    m_ui->italicCheckBox->setVisible(isVisble);
+    m_ui->fontHeadline->setVisible(isVisible);
+    m_ui->boldCheckBox->setVisible(isVisible);
+    m_ui->italicCheckBox->setVisible(isVisible);
+    m_ui->fontSpacer1->setVisible(isVisible);
+    m_ui->fontSpacer2->setVisible(isVisible);
 
     m_ui->boldCheckBox->setChecked(format.bold());
     m_ui->italicCheckBox->setChecked(format.italic());
@@ -307,12 +383,15 @@ void ColorSchemeEdit::updateUnderlineControls()
 
     QSignalBlocker comboBoxSignalBlocker(m_ui->underlineComboBox);
 
-    bool isVisble= formatDescription.showControl(FormatDescription::ShowUnderlineControl);
+    bool isVisible= formatDescription.showControl(FormatDescription::ShowUnderlineControl);
 
-    m_ui->underlineLabel->setVisible(isVisble);
-    m_ui->underlineColorToolButton->setVisible(isVisble);
-    m_ui->eraseUnderlineColorToolButton->setVisible(isVisble);
-    m_ui->underlineComboBox->setVisible(isVisble);
+    m_ui->underlineHeadline->setVisible(isVisible);
+    m_ui->underlineLabel->setVisible(isVisible);
+    m_ui->underlineColorToolButton->setVisible(isVisible);
+    m_ui->eraseUnderlineColorToolButton->setVisible(isVisible);
+    m_ui->underlineComboBox->setVisible(isVisible);
+    m_ui->underlineSpacer1->setVisible(isVisible);
+    m_ui->underlineSpacer2->setVisible(isVisible);
 
     m_ui->underlineColorToolButton->setStyleSheet(colorButtonStyleSheet(format.underlineColor()));
     m_ui->eraseUnderlineColorToolButton->setEnabled(!m_readOnly
@@ -391,6 +470,70 @@ void ColorSchemeEdit::eraseForeColor()
     }
 }
 
+void ColorSchemeEdit::changeRelativeForeColor()
+{
+    if (m_curItem == -1)
+        return;
+
+    double saturation = m_ui->foregroundSaturationSpinBox->value();
+    double lightness = m_ui->foregroundLightnessSpinBox->value();
+
+    for (const QModelIndex &index : m_ui->itemList->selectionModel()->selectedRows()) {
+        const TextStyle category = m_descriptions[index.row()].id();
+        m_scheme.formatFor(category).setRelativeForegroundSaturation(saturation);
+        m_scheme.formatFor(category).setRelativeForegroundLightness(lightness);
+        m_formatsModel->emitDataChanged(index);
+    }
+}
+
+void ColorSchemeEdit::changeRelativeBackColor()
+{
+    if (m_curItem == -1)
+        return;
+
+    double saturation = m_ui->backgroundSaturationSpinBox->value();
+    double lightness = m_ui->backgroundLightnessSpinBox->value();
+
+    for (const QModelIndex &index : m_ui->itemList->selectionModel()->selectedRows()) {
+        const TextStyle category = m_descriptions[index.row()].id();
+        m_scheme.formatFor(category).setRelativeBackgroundSaturation(saturation);
+        m_scheme.formatFor(category).setRelativeBackgroundLightness(lightness);
+        m_formatsModel->emitDataChanged(index);
+    }
+}
+
+void ColorSchemeEdit::eraseRelativeForeColor()
+{
+    if (m_curItem == -1)
+        return;
+
+    m_ui->foregroundSaturationSpinBox->setValue(0.0);
+    m_ui->foregroundLightnessSpinBox->setValue(0.0);
+
+    foreach (const QModelIndex &index, m_ui->itemList->selectionModel()->selectedRows()) {
+        const TextStyle category = m_descriptions[index.row()].id();
+        m_scheme.formatFor(category).setRelativeForegroundSaturation(0.0);
+        m_scheme.formatFor(category).setRelativeForegroundLightness(0.0);
+        m_formatsModel->emitDataChanged(index);
+    }
+}
+
+void ColorSchemeEdit::eraseRelativeBackColor()
+{
+    if (m_curItem == -1)
+        return;
+
+    m_ui->backgroundSaturationSpinBox->setValue(0.0);
+    m_ui->backgroundLightnessSpinBox->setValue(0.0);
+
+    foreach (const QModelIndex &index, m_ui->itemList->selectionModel()->selectedRows()) {
+        const TextStyle category = m_descriptions[index.row()].id();
+        m_scheme.formatFor(category).setRelativeBackgroundSaturation(0.0);
+        m_scheme.formatFor(category).setRelativeBackgroundLightness(0.0);
+        m_formatsModel->emitDataChanged(index);
+    }
+}
+
 void ColorSchemeEdit::checkCheckBoxes()
 {
     if (m_curItem == -1)
