@@ -89,9 +89,9 @@ public:
     void autoIndent(const QTextCursor &cursor, QChar typedChar = QChar::Null);
     void autoReindent(const QTextCursor &cursor);
     QTextCursor indent(const QTextCursor &cursor, bool blockSelection = false, int column = 0,
-                       int *offset = 0);
+                       int *offset = nullptr);
     QTextCursor unindent(const QTextCursor &cursor, bool blockSelection = false, int column = 0,
-                         int *offset = 0);
+                         int *offset = nullptr);
 
     TextMarks marks() const;
     bool addMark(TextMark *mark);
