@@ -39,6 +39,7 @@
 QT_BEGIN_NAMESPACE
 class QFileInfo;
 class QProcessEnvironment;
+class QToolBar;
 QT_END_NAMESPACE
 
 namespace Core { class Id; }
@@ -51,7 +52,7 @@ class VcsBaseClientSettings;
 class VcsJob;
 class VcsBaseClientImplPrivate;
 class VcsBaseClientPrivate;
-class VcsBaseEditorParameterWidget;
+class VcsBaseEditorConfig;
 
 class VCSBASE_EXPORT VcsBaseClientImpl : public QObject
 {
@@ -77,7 +78,7 @@ public:
                                          const QString &dynamicPropertyValue) const;
 
     VcsCommand *createCommand(const QString &workingDirectory,
-                              VcsBaseEditorWidget *editor = 0,
+                              VcsBaseEditorWidget *editor = nullptr,
                               JobOutputBindMode mode = NoOutputBind) const;
 
     void enqueueJob(VcsCommand *cmd, const QStringList &args,
@@ -103,6 +104,10 @@ public:
     Utils::SynchronousProcessResponse
     vcsFullySynchronousExec(const QString &workingDir, const QStringList &args,
                             unsigned flags = 0, int timeoutS = -1, QTextCodec *codec = nullptr) const;
+    Utils::SynchronousProcessResponse
+    vcsFullySynchronousExec(const QString &workingDir, const Utils::FileName &binary, const QStringList &args,
+                            unsigned flags = 0, int timeoutS = -1, QTextCodec *codec = nullptr) const;
+
 
     // Simple helper to execute a single command using createCommand and enqueueJob.
     VcsCommand *vcsExec(const QString &workingDirectory, const QStringList &arguments,
@@ -189,15 +194,14 @@ public:
 
     virtual QString findTopLevelForFile(const QFileInfo &file) const = 0;
 
+    virtual void view(const QString &source, const QString &id,
+                      const QStringList &extraOptions = QStringList());
+
 signals:
     void parsedStatus(const QList<VcsBase::VcsBaseClient::StatusItem> &statusList);
     // Passes on changed signals from VcsJob to Control
     void changed(const QVariant &v);
 
-public slots:
-    virtual void view(const QString &source, const QString &id,
-                      const QStringList &extraOptions = QStringList());
-
 protected:
     enum VcsCommandTag
     {
@@ -223,9 +227,9 @@ protected:
 
     virtual QStringList revisionSpec(const QString &revision) const = 0;
 
-    typedef std::function<VcsBaseEditorParameterWidget *()> ParameterWidgetCreator;
-    void setDiffParameterWidgetCreator(ParameterWidgetCreator creator);
-    void setLogParameterWidgetCreator(ParameterWidgetCreator creator);
+    typedef std::function<VcsBaseEditorConfig *(QToolBar *)> ConfigCreator;
+    void setDiffConfigCreator(ConfigCreator creator);
+    void setLogConfigCreator(ConfigCreator creator);
 
     virtual StatusItem parseStatusLine(const QString &line) const = 0;
 
