@@ -30,7 +30,7 @@
 #include "clangfilesystemwatcher.h"
 #include "codecompleter.h"
 #include "diagnosticset.h"
-#include "tokeninfos.h"
+#include "tokenprocessor.h"
 #include "clangexceptions.h"
 #include "skippedsourceranges.h"
 
@@ -57,11 +57,11 @@ namespace ClangBackEnd {
 ClangCodeModelServer::ClangCodeModelServer()
     : documents(projects, unsavedFiles)
 {
-    updateDocumentAnnotationsTimer.setSingleShot(true);
-    QObject::connect(&updateDocumentAnnotationsTimer,
+    updateAnnotationsTimer.setSingleShot(true);
+    QObject::connect(&updateAnnotationsTimer,
                      &QTimer::timeout,
                      [this]() {
-        processJobsForDirtyAndVisibleDocuments();
+        processJobsForVisibleDocuments();
     });
 
     updateVisibleButNotCurrentDocumentsTimer.setSingleShot(true);
@@ -75,7 +75,7 @@ ClangCodeModelServer::ClangCodeModelServer()
                      &ClangFileSystemWatcher::fileChanged,
                      [this](const Utf8String &filePath) {
         if (!documents.hasDocumentWithFilePath(filePath))
-            updateDocumentAnnotationsTimer.start(0);
+            updateAnnotationsTimer.start(0);
     });
 }
 
@@ -92,40 +92,39 @@ static std::vector<Document> operator+(const std::vector<Document> &a,
     return result;
 }
 
-// TODO: Rename to createOrUpdate...
-void ClangCodeModelServer::registerTranslationUnitsForEditor(const ClangBackEnd::RegisterTranslationUnitForEditorMessage &message)
+void ClangCodeModelServer::documentsOpened(const ClangBackEnd::DocumentsOpenedMessage &message)
 {
-    qCDebug(serverLog) << "########## registerTranslationUnitsForEditor";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::registerTranslationUnitsForEditor");
+    qCDebug(serverLog) << "########## documentsOpened";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::documentsOpened");
 
     try {
         DocumentResetInfos toReset;
         QVector<FileContainer> toCreate;
-        categorizeFileContainers(message.fileContainers(), toCreate, toReset);
+        categorizeFileContainers(message.fileContainers, toCreate, toReset);
 
         const std::vector<Document> createdDocuments = documents.create(toCreate);
         for (const auto &document : createdDocuments)
             documentProcessors().create(document);
         const std::vector<Document> resetDocuments_ = resetDocuments(toReset);
 
-        unsavedFiles.createOrUpdate(message.fileContainers());
-        documents.setUsedByCurrentEditor(message.currentEditorFilePath());
-        documents.setVisibleInEditors(message.visibleEditorFilePaths());
+        unsavedFiles.createOrUpdate(message.fileContainers);
+        documents.setUsedByCurrentEditor(message.currentEditorFilePath);
+        documents.setVisibleInEditors(message.visibleEditorFilePaths);
 
         processSuspendResumeJobs(documents.documents());
         processInitialJobsForDocuments(createdDocuments + resetDocuments_);
     } catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::registerTranslationUnitsForEditor:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::documentsOpened:" << exception.what();
     }
 }
 
-void ClangCodeModelServer::updateTranslationUnitsForEditor(const UpdateTranslationUnitsForEditorMessage &message)
+void ClangCodeModelServer::documentsChanged(const DocumentsChangedMessage &message)
 {
-    qCDebug(serverLog) << "########## updateTranslationUnitsForEditor";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::updateTranslationUnitsForEditor");
+    qCDebug(serverLog) << "########## documentsChanged";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::documentsChanged");
 
     try {
-        const auto newerFileContainers = documents.newerFileContainers(message.fileContainers());
+        const auto newerFileContainers = documents.newerFileContainers(message.fileContainers);
         if (newerFileContainers.size() > 0) {
             std::vector<Document> updateDocuments = documents.update(newerFileContainers);
             unsavedFiles.createOrUpdate(newerFileContainers);
@@ -138,27 +137,27 @@ void ClangCodeModelServer::updateTranslationUnitsForEditor(const UpdateTranslati
             // Start the jobs on the next event loop iteration since otherwise
             // we might block the translation unit for a completion request
             // that comes right after this message.
-            updateDocumentAnnotationsTimer.start(0);
+            updateAnnotationsTimer.start(0);
         }
     } catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::updateTranslationUnitsForEditor:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::documentsChanged:" << exception.what();
     }
 }
 
-void ClangCodeModelServer::unregisterTranslationUnitsForEditor(const ClangBackEnd::UnregisterTranslationUnitsForEditorMessage &message)
+void ClangCodeModelServer::documentsClosed(const ClangBackEnd::DocumentsClosedMessage &message)
 {
-    qCDebug(serverLog) << "########## unregisterTranslationUnitsForEditor";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::unregisterTranslationUnitsForEditor");
+    qCDebug(serverLog) << "########## documentsClosed";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::documentsClosed");
 
     try {
-        for (const auto &fileContainer : message.fileContainers()) {
+        for (const auto &fileContainer : message.fileContainers) {
             const Document &document = documents.document(fileContainer);
             documentProcessors().remove(document);
         }
-        documents.remove(message.fileContainers());
-        unsavedFiles.remove(message.fileContainers());
+        documents.remove(message.fileContainers);
+        unsavedFiles.remove(message.fileContainers);
     } catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::unregisterTranslationUnitsForEditor:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::documentsClosed:" << exception.what();
     }
 }
 
@@ -170,113 +169,114 @@ static DocumentResetInfos toDocumentResetInfos(const std::vector<Document> &docu
     return infos;
 }
 
-void ClangCodeModelServer::registerProjectPartsForEditor(const RegisterProjectPartsForEditorMessage &message)
+void ClangCodeModelServer::projectPartsUpdated(const ProjectPartsUpdatedMessage &message)
 {
-    qCDebug(serverLog) << "########## registerProjectPartsForEditor";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::registerProjectPartsForEditor");
+    qCDebug(serverLog) << "########## projectPartsUpdated";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::projectPartsUpdated");
 
     try {
-        projects.createOrUpdate(message.projectContainers());
+        projects.createOrUpdate(message.projectContainers);
         std::vector<Document> affectedDocuments = documents.setDocumentsDirtyIfProjectPartChanged();
 
         resetDocuments(toDocumentResetInfos(affectedDocuments));
 
-        processJobsForDirtyAndVisibleDocuments();
+        processJobsForVisibleDocuments();
     } catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::registerProjectPartsForEditor:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::projectPartsUpdated:" << exception.what();
     }
 }
 
-void ClangCodeModelServer::unregisterProjectPartsForEditor(const UnregisterProjectPartsForEditorMessage &message)
+void ClangCodeModelServer::projectPartsRemoved(const ProjectPartsRemovedMessage &message)
 {
-    qCDebug(serverLog) << "########## unregisterProjectPartsForEditor";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::unregisterProjectPartsForEditor");
+    qCDebug(serverLog) << "########## projectPartsRemoved";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::projectPartsRemoved");
 
     try {
-        projects.remove(message.projectPartIds());
+        projects.remove(message.projectPartIds);
     } catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::unregisterProjectPartsForEditor:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::projectPartsRemoved:" << exception.what();
     }
 }
 
-void ClangCodeModelServer::registerUnsavedFilesForEditor(const RegisterUnsavedFilesForEditorMessage &message)
+void ClangCodeModelServer::unsavedFilesUpdated(const UnsavedFilesUpdatedMessage &message)
 {
-    qCDebug(serverLog) << "########## registerUnsavedFilesForEditor";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::registerUnsavedFilesForEditor");
+    qCDebug(serverLog) << "########## unsavedFilesUpdated";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::unsavedFilesUpdated");
 
     try {
-        unsavedFiles.createOrUpdate(message.fileContainers());
-        documents.updateDocumentsWithChangedDependencies(message.fileContainers());
+        unsavedFiles.createOrUpdate(message.fileContainers);
+        documents.updateDocumentsWithChangedDependencies(message.fileContainers);
 
-        updateDocumentAnnotationsTimer.start(updateDocumentAnnotationsTimeOutInMs);
+        updateAnnotationsTimer.start(updateAnnotationsTimeOutInMs);
     } catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::registerUnsavedFilesForEditor:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::unsavedFilesUpdated:" << exception.what();
     }
 }
 
-void ClangCodeModelServer::unregisterUnsavedFilesForEditor(const UnregisterUnsavedFilesForEditorMessage &message)
+void ClangCodeModelServer::unsavedFilesRemoved(const UnsavedFilesRemovedMessage &message)
 {
-    qCDebug(serverLog) << "########## registerUnsavedFilesForEditor";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::unregisterUnsavedFilesForEditor");
+    qCDebug(serverLog) << "########## unsavedFilesRemoved";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::unsavedFilesRemoved");
 
     try {
-        unsavedFiles.remove(message.fileContainers());
-        documents.updateDocumentsWithChangedDependencies(message.fileContainers());
+        unsavedFiles.remove(message.fileContainers);
+        documents.updateDocumentsWithChangedDependencies(message.fileContainers);
     } catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::unregisterUnsavedFilesForEditor:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::unsavedFilesRemoved:" << exception.what();
     }
 }
 
-void ClangCodeModelServer::completeCode(const ClangBackEnd::CompleteCodeMessage &message)
+void ClangCodeModelServer::requestCompletions(const ClangBackEnd::RequestCompletionsMessage &message)
 {
-    qCDebug(serverLog) << "########## completeCode";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::completeCode");
+    qCDebug(serverLog) << "########## requestCompletions";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::requestCompletions");
 
     try {
-        Document document = documents.document(message.filePath(), message.projectPartId());
+        Document document = documents.document(message.filePath, message.projectPartId);
         DocumentProcessor processor = documentProcessors().processor(document);
 
-        JobRequest jobRequest = processor.createJobRequest(JobRequest::Type::CompleteCode);
-        jobRequest.line = message.line();
-        jobRequest.column = message.column();
-        jobRequest.funcNameStartLine = message.funcNameStartLine();
-        jobRequest.funcNameStartColumn = message.funcNameStartColumn();
-        jobRequest.ticketNumber = message.ticketNumber();
+        JobRequest jobRequest = processor.createJobRequest(JobRequest::Type::RequestCompletions);
+        jobRequest.line = message.line;
+        jobRequest.column = message.column;
+        jobRequest.funcNameStartLine = message.funcNameStartLine;
+        jobRequest.funcNameStartColumn = message.funcNameStartColumn;
+        jobRequest.ticketNumber = message.ticketNumber;
 
         processor.addJob(jobRequest);
         processor.process();
     }  catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::completeCode:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::requestCompletions:" << exception.what();
     }
 }
 
-void ClangCodeModelServer::requestDocumentAnnotations(const RequestDocumentAnnotationsMessage &message)
+void ClangCodeModelServer::requestAnnotations(const RequestAnnotationsMessage &message)
 {
-    qCDebug(serverLog) << "########## requestDocumentAnnotations";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::requestDocumentAnnotations");
+    qCDebug(serverLog) << "########## requestAnnotations";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::requestAnnotations");
 
     try {
-        auto document = documents.document(message.fileContainer().filePath(),
-                                           message.fileContainer().projectPartId());
+        auto document = documents.document(message.fileContainer.filePath,
+                                           message.fileContainer.projectPartId);
 
         DocumentProcessor processor = documentProcessors().processor(document);
-        processor.addJob(JobRequest::Type::RequestDocumentAnnotations);
+        processor.addJob(JobRequest::Type::RequestAnnotations);
+        processor.addJob(JobRequest::Type::UpdateExtraAnnotations);
         processor.process();
     }  catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::requestDocumentAnnotations:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::requestAnnotations:" << exception.what();
     }
 }
 
 template <class MessageType>
 static void fillJobRequest(JobRequest &jobRequest, const MessageType &message)
 {
-    jobRequest.line = message.line();
-    jobRequest.column = message.column();
-    jobRequest.ticketNumber = message.ticketNumber();
-    jobRequest.textCodecName = message.fileContainer().textCodecName();
+    jobRequest.line = message.line;
+    jobRequest.column = message.column;
+    jobRequest.ticketNumber = message.ticketNumber;
+    jobRequest.textCodecName = message.fileContainer.textCodecName;
     // The unsaved files might get updater later, so take the current
     // revision for the request.
-    jobRequest.documentRevision = message.fileContainer().documentRevision();
+    jobRequest.documentRevision = message.fileContainer.documentRevision;
 }
 
 void ClangCodeModelServer::requestReferences(const RequestReferencesMessage &message)
@@ -285,13 +285,13 @@ void ClangCodeModelServer::requestReferences(const RequestReferencesMessage &mes
     TIME_SCOPE_DURATION("ClangCodeModelServer::requestReferences");
 
     try {
-        const Document document = documents.document(message.fileContainer().filePath(),
-                                                     message.fileContainer().projectPartId());
+        const Document document = documents.document(message.fileContainer.filePath,
+                                                     message.fileContainer.projectPartId);
         DocumentProcessor processor = documentProcessors().processor(document);
 
         JobRequest jobRequest = processor.createJobRequest(JobRequest::Type::RequestReferences);
         fillJobRequest(jobRequest, message);
-        jobRequest.localReferences = message.local();
+        jobRequest.localReferences = message.local;
         processor.addJob(jobRequest);
         processor.process();
     }  catch (const std::exception &exception) {
@@ -305,12 +305,11 @@ void ClangCodeModelServer::requestFollowSymbol(const RequestFollowSymbolMessage
     TIME_SCOPE_DURATION("ClangCodeModelServer::requestFollowSymbol");
 
     try {
-        auto projectPartId = message.fileContainer().projectPartId();
-        Document document = documents.document(message.fileContainer().filePath(),
-                                                     projectPartId);
+        const Utf8String &projectPartId = message.fileContainer.projectPartId;
+        Document document = documents.document(message.fileContainer.filePath, projectPartId);
         DocumentProcessor processor = documentProcessors().processor(document);
 
-        JobRequest jobRequest = processor.createJobRequest(JobRequest::Type::FollowSymbol);
+        JobRequest jobRequest = processor.createJobRequest(JobRequest::Type::RequestFollowSymbol);
         fillJobRequest(jobRequest, message);
         processor.addJob(jobRequest);
         processor.process();
@@ -324,8 +323,8 @@ void ClangCodeModelServer::requestToolTip(const RequestToolTipMessage &message)
     TIME_SCOPE_DURATION("ClangCodeModelServer::requestToolTip");
 
     try {
-        const Document document = documents.document(message.fileContainer().filePath(),
-                                                     message.fileContainer().projectPartId());
+        const Document document = documents.document(message.fileContainer.filePath,
+                                                     message.fileContainer.projectPartId);
         DocumentProcessor processor = documentProcessors().processor(document);
 
         JobRequest jobRequest = processor.createJobRequest(JobRequest::Type::RequestToolTip);
@@ -338,19 +337,19 @@ void ClangCodeModelServer::requestToolTip(const RequestToolTipMessage &message)
     }
 }
 
-void ClangCodeModelServer::updateVisibleTranslationUnits(const UpdateVisibleTranslationUnitsMessage &message)
+void ClangCodeModelServer::documentVisibilityChanged(const DocumentVisibilityChangedMessage &message)
 {
-    qCDebug(serverLog) << "########## updateVisibleTranslationUnits";
-    TIME_SCOPE_DURATION("ClangCodeModelServer::updateVisibleTranslationUnits");
+    qCDebug(serverLog) << "########## documentVisibilityChanged";
+    TIME_SCOPE_DURATION("ClangCodeModelServer::documentVisibilityChanged");
 
     try {
-        documents.setUsedByCurrentEditor(message.currentEditorFilePath());
-        documents.setVisibleInEditors(message.visibleEditorFilePaths());
+        documents.setUsedByCurrentEditor(message.currentEditorFilePath);
+        documents.setVisibleInEditors(message.visibleEditorFilePaths);
         processSuspendResumeJobs(documents.documents());
 
-        updateDocumentAnnotationsTimer.start(0);
+        updateAnnotationsTimer.start(0);
     }  catch (const std::exception &exception) {
-        qWarning() << "Error in ClangCodeModelServer::updateVisibleTranslationUnits:" << exception.what();
+        qWarning() << "Error in ClangCodeModelServer::documentVisibilityChanged:" << exception.what();
     }
 }
 
@@ -371,23 +370,33 @@ int ClangCodeModelServer::queueSizeForTestsOnly()
 
 bool ClangCodeModelServer::isTimerRunningForTestOnly() const
 {
-    return updateDocumentAnnotationsTimer.isActive();
+    return updateAnnotationsTimer.isActive();
 }
 
-void ClangCodeModelServer::processJobsForDirtyAndVisibleDocuments()
+void ClangCodeModelServer::processJobsForVisibleDocuments()
 {
-    processJobsForDirtyCurrentDocument();
+    processJobsForCurrentDocument();
     processTimerForVisibleButNotCurrentDocuments();
 }
 
-void ClangCodeModelServer::processJobsForDirtyCurrentDocument()
+void ClangCodeModelServer::processJobsForCurrentDocument()
 {
-    auto currentDirtyDocuments = documents.filtered([](const Document &document) {
-        return document.isDirty() && document.isUsedByCurrentEditor();
+    auto currentDocuments = documents.filtered([](const Document &document) {
+        return document.isUsedByCurrentEditor()
+               && (document.isDirty() || document.documentRevision() == 1);
     });
-    QTC_CHECK(currentDirtyDocuments.size() <= 1);
+    QTC_CHECK(currentDocuments.size() <= 1);
 
-    addAndRunUpdateJobs(currentDirtyDocuments);
+    addAndRunUpdateJobs(currentDocuments);
+}
+
+static void addUpdateAnnotationsJobsAndProcess(DocumentProcessor &processor)
+{
+    processor.addJob(JobRequest::Type::UpdateAnnotations,
+                     PreferredTranslationUnit::PreviouslyParsed);
+    processor.addJob(JobRequest::Type::UpdateExtraAnnotations,
+                     PreferredTranslationUnit::RecentlyParsed);
+    processor.process();
 }
 
 void ClangCodeModelServer::addAndRunUpdateJobs(std::vector<Document> documents)
@@ -396,9 +405,7 @@ void ClangCodeModelServer::addAndRunUpdateJobs(std::vector<Document> documents)
         DocumentProcessor processor = documentProcessors().processor(document);
 
         // Run the regular edit-reparse-job
-        processor.addJob(JobRequest::Type::UpdateDocumentAnnotations,
-                         PreferredTranslationUnit::PreviouslyParsed);
-        processor.process();
+        addUpdateAnnotationsJobsAndProcess(processor);
 
         // If requested, run jobs to increase the responsiveness of the document
         if (useSupportiveTranslationUnit() && document.isResponsivenessIncreaseNeeded()) {
@@ -426,6 +433,10 @@ void ClangCodeModelServer::processSuspendResumeJobs(const std::vector<Document>
     for (const SuspendResumeJobsEntry &entry : suspendResumeJobs) {
         DocumentProcessor processor = documentProcessors().processor(entry.document);
         processor.addJob(entry.jobRequestType, entry.preferredTranslationUnit);
+        if (entry.jobRequestType == JobRequest::Type::ResumeDocument) {
+            processor.addJob(JobRequest::Type::UpdateExtraAnnotations,
+                             PreferredTranslationUnit::RecentlyParsed);
+        }
         processor.process();
     }
 }
@@ -436,7 +447,7 @@ void ClangCodeModelServer::categorizeFileContainers(const QVector<FileContainer>
 {
     for (const FileContainer &fileContainer : fileContainers) {
         const std::vector<Document> matching = documents.filtered([&](const Document &document) {
-            return document.filePath() == fileContainer.filePath();
+            return document.filePath() == fileContainer.filePath;
         });
         if (matching.empty())
             toCreate.push_back(fileContainer);
@@ -451,7 +462,7 @@ std::vector<Document> ClangCodeModelServer::resetDocuments(const DocumentResetIn
 
     for (const DocumentResetInfo &info : infos) {
         const Document &document = info.documentToRemove;
-        QTC_CHECK(document.filePath() == info.fileContainer.filePath());
+        QTC_CHECK(document.filePath() == info.fileContainer.filePath);
 
         documents.remove({document.fileContainer()});
 
@@ -473,15 +484,13 @@ void ClangCodeModelServer::processInitialJobsForDocuments(const std::vector<Docu
 {
     for (const auto &document : documents) {
         DocumentProcessor processor = documentProcessors().processor(document);
-        processor.addJob(JobRequest::Type::UpdateDocumentAnnotations);
-        processor.addJob(JobRequest::Type::CreateInitialDocumentPreamble);
-        processor.process();
+        addUpdateAnnotationsJobsAndProcess(processor);
     }
 }
 
-void ClangCodeModelServer::setUpdateDocumentAnnotationsTimeOutInMsForTestsOnly(int value)
+void ClangCodeModelServer::setUpdateAnnotationsTimeOutInMsForTestsOnly(int value)
 {
-    updateDocumentAnnotationsTimeOutInMs = value;
+    updateAnnotationsTimeOutInMs = value;
 }
 
 void ClangCodeModelServer::setUpdateVisibleButNotCurrentDocumentsTimeOutInMsForTestsOnly(int value)
